/**********************************
action.js
Library code to build - manipulate and query system 'actions'. This file
contains all the required generic code for an action model. There are 
2 classes:
  ActionSet : Defines a set of actions.
  Action    : An individual action.
The code is generic and associated with the 'community' software model. Actions 
may be assigned to individual members of the communtiy. A report can be obtained
describing which individuals have outstanding actions - and of what type.
**********************************/
var ActionHandlers;
// The constructor for an action takes *one* parameter - the node ID of the action to load. If this
// parameter is missing then a new, empty action is created.
function Action(id) {
  if (id!=null) {
    if (!isNaN(id)) {
      // Try to load the catalogue node that represents this element.
      var it = rb.catalogue.category.get({ID:id, idMkr:'N',allLevels:false,nextLevel:false});
      if (it.getNextRow()) {
        // Found it. Make sure it is an Action item by checking the parent.
        if (it.parentID==SysData.Categories.Actions) {
          // Copy the attributes.
          this.makeFromRecord(it);
        }
        else {
          // dumpObj(it, "ErrorNode");
          rb.debug.error("ERROR! Attempt to load an Action object from an inappropriate catalogue item");
        }
      }
    }
  }
  else {
    // Create an empty one.
    this.nodeData = {owner:-1, oldOwner:-1, empId:-1, oldEmpId:-1, dueDate:new Date(),title:'',status:'Outstanding', href:'', 
                     write:'super,admin', read:'super,admin',
                     type:'simple', yeslabel:'Accept', nolabel:'Reject', label:'Confirm'}
  }
}
Action.attribs = {nodeID:1, name:1, description:1, parentID:1, nodeData:1};
Action.completion = {};
Action.prototype.calcNameDate = function() {
  var now = this.nodeData.dueDate.valueOf();
  var end = new Date(2030, 12, 31);
  var key = (end.valueOf()-now.valueOf());
  this.name = rb.page.format("%014.0f",key);
}
// *Remove* an action. What actually happens is we simply rename the action so it becomes invisible.
Action.prototype.remove = function() {
  if (this.nodeID>0) {
    this.path[1]='$action$';
    this.description = '/'+this.path.join('/');
    this.save();
    // rb.catalogue.category.remove(this.nodeID);
  }
}
Action.prototype.makeFromRecord = function(record) {
  for (var i in Action.attribs) {
    if (record[i]!=null)
      this[i] = record[i];
    else
      delete this[i];
  }
  // Take the description and break it into path elements.
	this.path = this.description.split("/");
	if (this.path[0] == "")
		this.path.shift();

  if (this.nodeData==null)
    this.nodeData = {};

  this.nodeData.oldEmpId = -1;
  if (this.nodeData.empId!=null)
    this.nodeData.oldEmpId = this.nodeData.empId;

  this.nodeData.oldOwner = -1;
  if (this.nodeData.owner!=null) {
    this.nodeData.oldOwner = this.nodeData.owner;
  }

  // Auto generate a name. This allows records to be viewed most recent first.
  if (this.nodeData.dueDate==null)
    this.nodeData.dueDate = new Date;

  if (this.nodeData.status==null)
    this.nodeData.status = 'Outstanding';

  if (this.nodeData.title==null)
    this.nodeData.title = 'Untitled';

  if (this.nodeData.href==null)
    this.nodeData.href = '';

  if (this.nodeData.read==null)
    this.nodeData.read = 'super,admin';
  
  if (this.nodeData.write==null)
    this.nodeData.write = 'super,admin';

  if (this.nodeData.yeslabel==null)
    this.nodeData.yeslabel = 'Accept';

  if (this.nodeData.nolabel==null)
    this.nodeData.nolabel = 'Reject';

  if (this.nodeData.label==null)
    this.nodeData.label = 'Confirm';

  this.nodeData.status = (this.path[3]=='C'?'Complete':(Date()>this.nodeData.dueDate?'Overdue':'Outstanding'));
  this.calcNameDate();
}

// Create a new node or update an existing action.
Action.prototype.save = function() {
var res = false;
  rb.debug.write("<h1>INSIDE ACTION.save() METHOD!!!</h1>\n");

  // Path *must* be unique - if not then replace existing record.
  if (this.nodeID==null || this.nodeID<0) {
    var srch = {ID:SysData.Categories.Actions, idMkr:'N', keyword:[this.description],searchInDescription:true, nextLevel:1, allLevels:1}

    var it = rb.catalogue.category.get(srch);
    if (it.getNextRow()) {
      // Already exists - replace existing entry.
      rb.page.write("<h1>FOUND EXISTING ACTION - REPLACING ('"+this.description+"'</h1>\n");
      this.nodeID = it.nodeID;
    }
  }

  // If there is an 'ID' parameter and it's value is >0 - save. Otherwise create.
  if (this.nodeID!=null && this.nodeID>0) {
    // Save existing element.
    rb.debug.write("<h4>UPDATING EXISTING ENTRY</h4>");
    res = (rb.catalogue.category.modify(this)>0);

    // If the empId has changed then make the 
    if (this.nodeData.empId != this.nodeData.oldEmpId) {
      // It's changed. Remove linkage from old contact to the new one.
      if (this.nodeData.oldEmpId>0)
        res &= (rb.contact.unlinkCatNode(this.nodeData.oldEmpId, this.nodeID)>0);
      
      if (this.nodeData.empId>0)
        res &= (rb.contact.linkCatNode(this.nodeData.empId, this.nodeID)>0);
    }
    if (this.nodeData.owner != this.nodeData.oldOwner) {
      rb.debug.write("<b>Creating link to owner ID="+this.nodeData.ownerID+"</b><br>");
      // They are different.
      if  (this.nodeData.oldOwner != this.nodeData.empId && this.nodeData.oldOwner>0) {
        rb.debug.write("<b>unlinking</b><br>");
        res &= (rb.contact.unlinkCatNode(this.nodeData.oldOwner, this.nodeID)>0);
      }          
      if (this.nodeData.owner>0 && this.nodeData.owner!=this.nodeData.empId) {
        rb.debug.write("<b>linking</b><br>");
        res &= (rb.contact.linkCatNode(this.nodeData.owner, this.nodeID)>0);
      }
    }
  }
  else {
    rb.debug.write("<h4>NEW ACTION ENTRY</h4>");
    this.parentID = SysData.Categories.Actions;
    this.nodeID = rb.catalogue.category.create(this);
    res = (this.nodeID>0);
    if (res) {
      if (this.nodeData.empId>0)
        res = (rb.contact.linkCatNode(this.nodeData.empId, this.nodeID)>0);
      rb.debug.write("<h4>OwnerID = "+this.nodeData.owner+"</h4>\n");
      if (this.nodeData.owner>0 && this.nodeData.owner!=this.nodeData.empId) {
        rb.debug.write("<h4>OwnerID set for action - linking to owner:"+this.nodeData.owner+"</h4>\n");
        res &= (rb.contact.linkCatNode(this.nodeData.owner, this.nodeID)>0);
      }
    }
  }
  return res;
}

// If the action is *not* already complete, make it so, store the completion date and the
// user ID of the person currently logged into the system (make sure they have access rights first!)
Action.prototype.makeComplete = function(outcome) {
var res = false;

  // Need to find an 'outcome' that matches that specified in the parameter, then apply that to the item.
  if (this.nodeID>0 && this.nodeData.status != 'Complete') {
    if (this.nodeData.write==null || extauth.isIn(this.nodeData.write)) {
      for (var i=0;i<this.nodeData.outcome.length;i++) {
        var oc = this.nodeData.outcome[i];
        if (oc.name==outcome) {
          // Completing this one.
          rb.debug.write("Completing action - outcome='"+outcome+"'......<br>\n");
          var allowComplete = true;

          this.nodeData.result = outcome;
          // If there is a completion action - try and run it now.
          if (this.nodeData.oncomplete && ActionHandlers[this.nodeData.oncomplete]) {
            // Execute the action - if it exists.
            rb.debug.write("Executing action handler<br>\n");
            var fn = ActionHandlers[this.nodeData.oncomplete];
            res = allowComplete = fn(this, oc);

          }
          if (allowComplete) {
            this.nodeData.status       = 'Complete';
            this.nodeData.completeDate = new Date();
            this.nodeData.completeUser = extauth.sessionUser;

            // Change the path name and save.
            this.path[3] = 'C';
            this.description = '/'+this.path.join('/');
            this.save();
          }
          break;
        }
      }
    }
  }
  return res;
}

// Meta-function that takes a todoID, loads the description and then makes it complete.
Action.makeComplete = function(todoID, result) {
  var thisAction = new Action(todoID);
  thisAction.makeComplete(result);
}

// Constructor for an 'ActionSet'. Takes an optional 'action' parameter which is a path
// used to scope the request to a subset of the actions in the database. Actions are 
// retrieved through the 'getNextRow' method as with normal iterators. This class is 
// derived from 'Action'. 
function ActionSet(data) {
  var srch = {keyword:['/%/action/%'], ID:SysData.Categories.Actions, idMkr:'N', searchInDescription:true, allLevels:true, sortByName:true};

  if (data) {
    if (data.path)
      srch.keyword[0] = data.path;
    else {
      srch.year    = (data.year==null)?'%':data.year;
      srch.subtype = (data.subtype==null)?'%':data.subtype;
      srch.status  = (data.status==null)?'_':data.status;
      srch.keyword[0] = '/'+srch.year+'/action/'+srch.subtype+'/'+srch.status+'/%';
    }
    if (data.empId) {
      srch.ID = data.empId;
      srch.idMkr = 'C';
    }
  }

  // First call - create the iterator.
  this.rb$iterator = rb.catalogue.category.get(srch);
}
ActionSet.prototype = new Action;

// 'get 
ActionSet.prototype.getNextRow = function() {

  do {
    var res = this.rb$iterator.getNextRow();
    if (res) {
      // Visibility check? Is this action visible to the currently logged in person?
      if (this.rb$iterator.nodeData==null || extauth.isIn('super,'+this.rb$iterator.nodeData.write)) {
        // Yes - can see this record. Return.
        this.makeFromRecord(this.rb$iterator);
        return true;
      }
    }
  } while (res)
  return false;
}

