/********************************************************************************
function Event
Event class. Initialised with the data available from 
********************************************************************************/
function Event(input_data) {
  if (input_data == null)
    input_data = {};
   
  var id = (input_data.ID == null)?-1:input_data.ID;
   
  // If there is an ID then read the record.
  if (input_data.ID) {
    var commRecord = rb.contact.communityInfo(input_data.ID);

    if (commRecord == null)
      commRecord = {customData:{}};
  }
  this.makeEvent(commRecord);

  // Overlay the new parts - if any.
  this.customData = {};
  for (var i=0;i<Event.props.length;i++) {
    if (input_data[Event.props[i]]!=null)
      this[Event.props[i]] = input_data[Event.props[i]];
  }
  if (input_data.date)
    this.date = Util.parseDate(input_data.date);
  
  if (input_data.closeDate)
    this.closeDate = Util.parseDate(input_data.closeDate);

}

// Props copied from custom data.
Event.props = ['title','date','enddate','summary','description',
               'address1','address2','address3','address4',
               'organiser','regLevel','closeDate','readAccess','writeAccess',
               'assocParent'  // Defined if I want this under *another* category as well as the event category.
              ];

Event.prototype.makeEvent = function(community) {
  if (community==null)
    community={};
  
  var srcData = community.customData;
  if (srcData)
    srcData = srcData.eventData;

  if (srcData) {
    // Do the copy.
    for (var i=0;i<Event.props.length;i++) {
      if (srcData[Event.props[i]]!=null)
        this[Event.props[i]] = srcData[Event.props[i]];
      else 
        delete this[Event.props[i]];
    }
  }
  this.ID = community.ID;
}

// Return the number of days of this event. Inclusive of end-days.
Event.prototype.calcDuration = function() {
  dumpObj(this, "this");
  if (this.enddate==null || this.enddate=='')
    this.enddate=this.date;

  var d1 = new Date(this.date);
  var d2 = new Date(this.enddate);

  var diff = d2.valueOf() - d1.valueOf();

  // Divide by 24 hours.
  diff = Math.floor(diff/(24*3600000))+1;
  rb.debug.write("<h1>Difference between "+d1.valueOf()+" and "+d2.valueOf()+" is "+diff+"</h1>\n");

  // wind one back to start of day - one
  return diff;
}

// calcKey()
// generate a 'name' key for this event such that when sorted in name order - new items appear first.
// this is done by making the key some date in the future minus the event date.
Event.prototype.calcKey = function() {
  var now = this.date.valueOf()
  var end = new Date(2030, 12, 31);
  var key = (end.valueOf()-now.valueOf());
  return rb.page.format("%014.0f",key);
}

Event.prototype.makeCommunity = function() {
  var comm = {};

  // Copy the entire dataset to the customData - then copy a few bits elsewhere.
  comm.customData  = this.customData;
  comm.description = this.title;
  comm.name        = this.calcKey();
  comm.ID          = this.ID;

  // Copy all other known properies to the custom data area.
  if (comm.customData.eventData==null)
    comm.customData.eventData = new Object;

  for (var i=0;i<Event.props.length;i++) {
    if (this[Event.props[i]]!=null)
      comm.customData.eventData[Event.props[i]] = this[Event.props[i]];
  }
  return comm;
}

/********************************************************************************
Event.prototype.create
Use the data in the membership object to create a new contact record to represent
the new member. The function returns -1 if it fails.
********************************************************************************/
Event.prototype.create = function () {

    // Create the real contact record by copying fields from this object.
    var newContact = this.makeCommunity();
    this.ID        = -1;

    if (newContact) {
      this.ID = rb.contact.createCommunity(newContact);
      if (this.ID>=0) {
        // The event has been created - link it to the event catalogue node.
        rb.contact.linkCatNode(this.ID, SysData.RootCats.Events);

        // Also create a catalogue item (don't really want to!) as a way of 
        // attaching this event to the organisers.
        var catId = rb.catalogue.item.create({code:this.ID, description:'Event', itemData:{event:{communityID:this.ID}}});

        if (catId>=0) {
          // Attach this item under the community so we can find it.
          rb.contact.linkCatItem(this.ID, catId);
          rb.catalogue.category.linkItem(catId, SysData.RootCats.Events);

          // *If* there is an 'assocParent' defined then also link under that node.
          if (this.assocParent>0) {
            // This parameter came across the web - make sure it is a valid event ID.
            var valIt = rb.catalogue.category.get({ID:this.assocParent, idMkr:'N', allLevels:0, nextLevel:0});
            if (valIt.getNextRow()) {
              rb.contact.linkCatNode(this.ID, this.assocParent);
              rb.catalogue.category.linkItem(catId, this.assocParent);
            }
          }
          // Now - need to make the organisers of this event own the catalogue item.
          for (var i=0;i<this.organiser.length;i++) {
            // Have an organiser uName - get the corresponding ID then link.
            var contact = rb.contact.individualInfo(-1, this.organiser[i]);
            if (contact)
              rb.contact.linkCatItem(contact.ID, catId);
          }
        }
      }
    }
    return this.ID;
}


/********************************************************************************
Event.prototype.save
This only thing we change is the community data - everything else is a key to 
that data.
********************************************************************************/
Event.prototype.save = function () {
  var newCommunity = this.makeCommunity();
  var res          = -1;
  
  if (newCommunity) {
    // rb.debug.write("<h1>Event being asked to save : </h1>");
    // dumpObj(newCommunity, "dataToSave");
    res = rb.contact.modifyCommunity(newCommunity);
  }
  return res;
}

/********************************************************************************
Event.prototype.remove
Delete an event. We never actually delete anything - instead we move it to the
'deleted' branch.
********************************************************************************/
Event.prototype.remove = function () {

}

/********************************************************************************
registerUser against 'this' event. Several steps:
1. Add to the community that represents the event.
   This allows the query 'who has registered for this event'

********************************************************************************/
Event.prototype.registerUser = function (userId) {
var res = false;

  if (this.ID>0) {
    // OK - have the community representing the event. Join the individual.
    rb.contact.link(userId, this.ID);
    res = true;
  }
  return res;
}
/********************************************************************************
EventIt - Event Iterator.
This is a new JavaScript class - it isn't directly instantiated by the application
instead being returned from the Event.get class method. The class 'looks' exactly
like an iterator - but behind the scenes does the manipulation to extract contact
info into the member structure.

Because an iterator is basically a subclass of the basic data structure - with the
addition of the iterator methods. In JavaScript we achieve this simply by setting
the prototype to be an instance of Event.
 ********************************************************************************/
/********************************************************************************
EventIt
Iterator for member information. This is the constructor. Doesn't do anything 
except at the moment.
********************************************************************************/
function EventIt(contactIterator) {
    this.rb$Iterator = contactIterator;
}

/********************************************************************************
* Force the iterator to be a subclass of the Event.
*********************************************************************************/
EventIt.prototype = new Event;

Event.isEvent = function (community) {
  return true;
}
/********************************************************************************
EventIt.prototype.getNextRow
Overload for the iterator getNextRow method.
********************************************************************************/
EventIt.prototype.getNextRow = function () {
  var res;
  
  do {
    res = this.rb$Iterator.getNextRow();
  } while (res && !Event.isEvent(this.rb$Iterator));
  
  // Copy the data from the iterator to 'this'.
  if (res) {
    // Load the member fields from the iterator.
    this.makeEvent(this.rb$Iterator);
  }
  return res;
}
/********************************************************************************
Event.get
Create an iterator for members.
********************************************************************************/
Event.get = function (catNodeId) {
    var iterator;
    var memIt;

    keywords = [];
    // Force the correct set of flags for finding individuals.
    flags = new Object;

    flags.searchCommunity     = true;
    flags.searchOU            = false;
    flags.searchIndividual    = false;
    flags.searchName          = false;
    flags.searchDescription   = true;
    flags.hasNodeID           = catNodeId;
    flags.sort                = 'name'

    dumpObj(flags,"EventSearchFlags");
    iterator = rb.contact.get(keywords, flags);
    memIt    = new EventIt(iterator);
    return memIt;
}



