// Library functions to test and resolve image references in an HTML document. Presented in
// the form of a ImgProc object with a 'resolve' method that takes an HTML document. The 
// document is parsed and image tags located. The src for each image tag is extracted and a test
// made on the local server for a file with that reference. If the file exists the image
// src is modified. If it is *not* then the tag reference is recorded as unresolved and
// returned to the caller.
var ImgProc = new Object;

// .parse
// Method to take an XML tree and to look for images that don't exist - or that exist within the database.
// 1. Real files - src attribute unchanged.
// 2. Database files - src attribute modified to an image redirection .rhtm file
// 3. Unknown images - src unchanged - entry added to 'unres' array and returned to the caller.
//
// If the method returns 'null' then all images are resolved. If there are unsresolved images
// the method returns an array of the 'img' tags that are unresolved. 
ImgProc.parse = function(parser, xmlTree, docId) {

  rb.debug.write("<h1>EXECUTING IMAGE VALIDATION</h1>\n");
  // Find all image tages.
  var imgTags = xmlTree.find('img');
  var unres   = [];

  dumpObj(imgTags,"imgTags", 3);

  for (var i=0;i<imgTags.length;i++) {
    // If it has a 'src' attribute - validate that it exist.
    var thisImg = imgTags[i];
    var tryResolve;
    var path='';
    
    if (thisImg.attributes.src != null) {
      var src = new String(thisImg.attributes.src);

      rb.debug.write("<p>src{1} = "+src+"<br>\n");
      if (src.indexOf('http://')==0) {
        src = src.slice(7);
        rb.debug.write("src{2} = "+src+"<br>\n");
        // Starts with 'http://' - so it's an href. See whether the target is this site.
        rb.debug.write("Server name is '"+rb.page.env.ServerName()+"'<br>\n");
        if (src.indexOf(rb.page.env.ServerName())<0) {
          // *not* a reference to this site - so assume it's a remote server.
          continue;
        }
        src = src.slice(rb.page.env.ServerName().length);
        rb.debug.write("src{3} = "+src+"<br>\n");

        // If the request starts with '/doc/image.rhtm' then this is a previously
        // resolved reference to a database located image file.
        if (src.indexOf('/doc/image.rhtm')==0) {
          // Previously resolved image. Validate here.
          path = src.slice(15);
          tryResolve = true;
        }
        else {
          // A file name. *Might* be an image in the file system - or one in the database already. If *not* then it's unresolved.
          tryResolve = true;
          path       = src;
        }
      }
      else if (src[0]=='/') {
        // Absolute path
        path = src;
        tryResolve = true;
      }
      else {
        var colonOffset = src.lastIndexOf(':');
        if (colonOffset>=0) {
          path = src.slice(colonOffset+1);
          tryResolve = true;
        }
      }
      if (tryResolve) {
        // Reduce the path to simply the filename part.
        rb.debug.write("<h1>Searching for image file:"+path+"</h1>\n");
        path = path.replace(/\\/g,'/');
        var lastSlash = path.lastIndexOf('/');
        if (lastSlash>0)
          path = path.slice(lastSlash+1);

        var fBin = new Binary;

        // Try the various places where we might expect to find the image.
        var found = (fBin.load('/images/'+path));
        if (!found)
          found = (fBin.load('/content/images/'+path));
        
        if (!found) {
          // Not a file - check the database.
          if (docId!=null & docId>0) {
            // Search data records associated with this document.
            var it = rb.catalogue.item.get({idMkr:'N', ID:SysData.RootCats.Images, searchInCode:true, keyword:[''+docId+'/'+path], allLevels:false});
            if (it.getNextRow()) {
              // The image is already in the database. Replace the 'src' attribute with a resolver reference.
              rb.debug.write("<h1>RESOLVED '"+thisImg.attributes.src+"' to '");
              path = '/doc/image.rhtm/'+docId+'/'+path;
              rb.debug.write(path+"'</h1>\n");
              thisImg.attributes.src = path;
            }
            else {
              rb.debug.write('<h4>Unresolved image "'+thisImg.attributes.src+'"</h4>\n');
              unres.push({tag:thisImg, path:path});
            }
          }
        }
      }
    }
  }
  dumpObj(unres, "unres");
  return unres.length>0?unres:null;
}

// Given a filename - check whether the file exists - either in the main database or the file system.
ImgProc.TestForFile = function(path) {
  var fBin = new Binary;
  var res = fBin.load(path);
  if (!res) {
    // Not a file - check the database.
    var it = rb.catalogue.item.get({idMkr:'N', ID:SysData.RootCats.Images, searchInCode:true, keyword:[path], allLevels:false});
    res = it.getNextRow();
  }
  return res;
}

// Take a path name and an image file (binary object) and save in the catalogue. If the file
// already exists in the database - return 'false'. Otherwise create the file and return 'true';
ImgProc.create = function (pathName, dataFile) {
  var res = ImgProc.TestForFile(pathName);
  if (res)
    return {path:pathName, msg:'File already exists'};

  if (dataFile.length>Doc.maxFileLength)
    return {path:pathName, msg:'File too large. Maximum filesize is '+Doc.maxFileLength/1024+'Kbytes'};

  // Create the new element.
  var fileRec = {name:'image file', owner:'site', description:pathName, fileDef:dataFile};
  var fileId  = rb.file.create(fileRec);
  if (fileId>0) {
    // Create the catalogue item and link under the images node.
    var itemId = rb.catalogue.item.create({code:pathName, description:'', fileRefs:[{fileOrder:1, fileID:fileId}]});
    if (itemId>0) {
      // Link this under the images branch.
      rb.catalogue.category.linkItem(itemId, SysData.RootCats.Images);
    }
  }
  return;
}