/*********************************************************
Component:    Redbourne Membership Framework.
Filename:     interest.js
Author:       Peter Wilson
**********************************************************
Overview
========
This file contains the JavaScript implementation of the
member interest facility. This is implemented as a number
of additional methods on the Member class and on objects
of that class. There are two main concepts to consider
in this implementation:
   1. Each 'interest' has a user friendly name.
   2. Each interest has an automatically created shortform
      code. This is as short as possible and is not meant 
      to be readable. Instead the interest code is stored 
      in the keywords list for members that share that 
      interest.

A JavaScript class encapuslates the 'interest' model,
each interest being represented in the template as a 
community. At the moment there isn't a lot of need
for this given the searchability of the description
field in the member template.

All template communities used to represent interests
have a 'rb$int$' prepended to the uname. The rest of
the uname is the shortform name for the interest group.
**********************************************************
$Id: interest.js,v 1.1 2002/05/15 11:56:04 petew Exp $
**********************************************************
$Log: interest.js,v $
Revision 1.1  2002/05/15 11:56:04  petew
Initial CVS Checkin.

Revision 1.1  2002/02/14 07:49:48  petew
Inital XPRL checkin - nothing finished yet!

Revision 1.4  2001/10/26 14:55:02  rob
*** empty log message ***

Revision 1.3  2001/07/19 13:03:25  rob
Member interface changes post ISOC implementation
Added event management membership class

Revision 1.1  2001/07/17 19:28:03  rob
new implementation

Revision 1.2  2001/05/02 15:45:47  steved
Inverted test fixed

Revision 1.1  2001/04/19 09:41:24  petew
First rev of the 'formal' membership code.

**********************************************************
Copyright (c) Redbourne Inc, 2001
**********************************************************/

var rb$MemberInterest = new Object;
rb$MemberInterest.props = [
        "ID",
        "uName", 
        "name",
        "description",
        "customData"
];
/********************************************************************************
Interest
Constructor for the JavaScript 'Interest' class. This takes an optional
'input_data' parameter that contains some initial data for the interest group. 
IF PRESENT the method tries to initialise the object as follows:

    Attempt to locate a community with the ID or uname in input_data
    Copy the input_data methods over the top of the resulting object.
********************************************************************************/
function Interest(input_data) {

   // Clear the loaded flag to indicate we don't have sync with the template.
   this.rb$loaded = false;

   // Set the ID to -1 to indicate that we don't know how to get the data -
   // No mapping exists to the contact template.
   this.ID = -1;


    // Pre-create an empty meta-data container.
   this.customData = new Object;

   // Create attributes used to manage meta-data. The basic
   // aim here is to make the member structure look the same as
   // the basic contact record - and hide the complexity - so
   // the original 'customData' field is still used by Pattern
   // Pages - but the field past to the contact template 
   // is the 'meta' field.
   if (input_data != null) {
      if (input_data.ID != null && input_data.ID != "")
         this.get(input_data.ID);
      else if (input_data.uName!=null && input_data.uName!="")
         this.get(-1, input_data.uName);

      // Now overlay the properties from the 'input_data' object. Use
      // the generic utility from the Member class.
      this.makeInterest(input_data);
   }
}


/****************************************************************
rb$memint$checkLoad - private
This utility method checks that the interest group is loaded 
from the contact template. If it isn't then an attempt is
made to load it.

The object must have a valid ID or uName.

The method returns 'true' if the interest group is loaded already
or is successfully loaded. If it can't be loaded then the method
returns false.
*****************************************************************/
Interest.prototype.checkLoad = function rb$memint$checkLoad() {

    if (!this.rb$loaded) {
       // Interest group is NOT already loaded!
       if (this.ID >=0 || this.uNake!=null) {
          // Try top load it now.
          this.get(this.ID, this.uName);
       }
    }
	 return this.rb$loaded;
}

/****************************************************************
rb$memint$makeCommunity - private
This method builds a community record from the current contents
of the interest object. The resulting object is returned to the
caller.
*****************************************************************/
Interest.prototype.makeCommunity = function () {
   // Create the real contact record by copying fields from this object.
   var newCommunity = new Object;

   // Copy relevant fields from 'this' to the new community record.
   Member.copyProperties(this, rb$MemberInterest.props, newCommunity);

   // Update the name field so it has a 'rb$int$' prefix.
   newCommunity.name = 'rb$int$'+newCommunity.name;
   
   return newCommunity;
}

/*****************************************************************
rb$memint$makeInterest - private
This method takes a community record and uses it to initialse the
interest record.
*****************************************************************/
Interest.prototype.makeInterest = function (aCommunity) {

   Member.copyProperties(aCommunity, rb$MemberInterest.props, this);

   // Strip the prefix from the head of the name.
   if (aCommunity.name!=null && (aCommunity.name.indexOf('rb$int$'))==0)
       this.name = aCommunity.name.substring(7, aCommunity.name.length);
}

/********************************************************************************
rb$memint$create
Use the data in the membership object to create a new contact record to represent
the new member. The function returns -1 if it fails.
********************************************************************************/
Interest.prototype.create = function () {

   // Create the real contact record by copying fields from this object.
   var newCommunity = this.makeCommunity();

   this.ID = rb.contact.createCommunity(newCommunity);
	return this.ID;
}

/********************************************************************************
rb$memint$save
Save the contents of the membership record to the contact template. This record
must already exist. 

This function exists as aMember.save();
********************************************************************************/
Interest.prototype.save = function () {
   // Create the real contact record by copying fields from this object.
   var aCommunity = this.makeCommunity();
   var res        = rb.contact.modifyCommunity(aCommunity);

   rb.debug.write('<p>rb.contact.modifyCommunity returned '+res+'</p>\n');
   return res;
}

/********************************************************************************
rb$memint$remove
Remove the member from the contact record.
********************************************************************************/
Interest.prototype.remove = function () {
    // First of all - see whether this interest group has an associated message board.
    if (!this.checkLoad()) {
        rb.debug.error("Can't load member information - remove request denied!");
        return -1;
    }

    // Now - if that worked.
    if (this.customData && this.customData.msgBoardName) {
       rb.debug.write('<h2>Interest group has a message board called : '+interestGroup.customData.msgBoardName+'</h2>');

       rb.message.group.removeGroup(this.customData.msgBoardName);
    }
    return rb.contact.remove(this.ID);
}

/********************************************************************************
rb$memint$get
Method in the Member component that populates the Member record from the specified
contact ID, or uname.

This function exists as aMember.get();
********************************************************************************/
Interest.prototype.get = function (id, uname) {
var aCommunity;

   rb.debug.write('<p>Called Interest.prototype.get with<br><blockquote>id='+id+'<br>uname='+uname+'</blockquote>');

   // ONE of id or uname must be specified.
   if (id<0 && uname==null) {
      rb.debug.error("Missing Member ID or uname! Invalid");
      return;
   }

   aCommunity = rb.contact.communityInfo(id, uname);

   if (aCommunity!=null) {
      this.makeInterest(aCommunity);
   }
   this.rb$loaded = true;
}

/********************************************************************************
rb$memint$createMsgBoard
This method will create a message board for this interest group. The caller can
pass in a name for the message board. If the name is not providedd then the 
method chooses a default name based on the uni	que ID of the interest group
for which the message board is required.

The routine returns either nothing (null) if everything worked OK,
or a text error message explaining why the operation failed.
********************************************************************************/
Interest.prototype.createMsgBoard = function (groupName) {

   // First thing to do is make sure we can load the interest group. 
	if (!this.checkLoad()) {
      return "Invalid Interest Group Object - doesn't exist!";
   }

   // First of all - see whether this group has a message board.
   var group = new MessageGroup(this.customData.msgBoardName);

	if (group.exists)
      return 'Interest group already has a message board attached!';

   // If a name has been provided, validate that it actually exists.
   rb.debug.write('<h2>Interest Group Name is '+groupName+'</h1>');
   if (groupName==null) 
      groupName = 'interest-'+interestId.toString();

   rb.debug.write('<h2>Interest Group Name is '+groupName+' - After</h1>');

   // Store the name in the meta-data for this object.
   this.customData.msgBoardName = groupName;

   // And save the interest group back to the contact template to make sure it doesn't get lost.
   this.save();

   // Create the new message board.
   group.name = groupName;

	return group.create();
}

/********************************************************************************
rb$memint$hasMsgBoard
Return the name of an associated message board if one exists for this interest 
group. If hte interest group has no message board then this method returns
a null object.
********************************************************************************/
Interest.prototype.hasMsgBoard = function () {

   if (!this.checkLoad())
	   return;

   if (this.customData.msgBoardName) {
      if (MessageGroup.getStatus(this.customData.msgBoardName)!=null)
         return this.customData.msgBoardName;

      // doesn't exist so delete name.
      delete this.customData.msgBoardName;
   }
}

/********************************************************************************
InterestIt
Iterator for interest information. This is the constructor. Doesn't do anything 
except at the moment.
********************************************************************************/
function InterestIt(contactIterator) {
   this.rb$Iterator = contactIterator;
}
InterestIt.prototype = new Interest;

/********************************************************************************
rb$memint$getNextRow
Overload for the iterator getNextRow method.
********************************************************************************/
InterestIt.prototype.getNextRow = function () {
var res;


   while ((res=this.rb$Iterator.getNextRow()) && (this.rb$Iterator.name.indexOf('rb$int$'))!=0);

   // Copy the data from the iterator to 'this'.
   if (res ) {
       // Remove all existing properties.
       Member.removeProperties(rb$MemberInterest.props, this)

       // Load the member fields from the iterator.
       this.makeInterest(this.rb$Iterator);
   }
   return res;
}
/********************************************************************************
rb$memint$setNextRow
Overload for the iterator setNextRow method. Basically a NOP.
********************************************************************************/
InterestIt.prototype.setNextRow = function (index) {
   return this.rb$Iterator.setNextRow(index);
}
/********************************************************************************
rb$memint$numberOfRows
Return the number of rows - returns the iterator value.
********************************************************************************/
InterestIt.prototype.numberOfRows = function (index) {
   return this.rb$Iterator.numberOfRows();
}
/********************************************************************************
rb$memint$currentRow
Return the number of rows - returns the iterator value.
********************************************************************************/
InterestIt.prototype.currentRow = function (index) {
   return this.rb$Iterator.currentRow();
}
/********************************************************************************
rb$memint$search
Create an iterator for members.
********************************************************************************/
Interest.get = function rb$memint$search(keywords, flags) {
var iterator;
var intIt;

    if (flags==null)
        flags = new Object;

    // Force the correct set of flags for finding individuals.
    flags.searchIndividual = false;
    flags.searchCommunity  = true;
    flags.searchOU         = false;
    flags.searchName       = true;
    flags.sort             = 'ID';

    iterator  = rb.contact.get(keywords, flags);
    intIt     = new InterestIt(iterator);
    return intIt;
}

/********************************************************************************
rb$memint$findMembers
Find members that are interested in at least one - or all of the 
interests in the arary. 
********************************************************************************/
Interest.findMembers = function (interestArray, inAll) {
var iterator;
var intIt;

    if (inAll==null)
        inAll = false;

    flags = new Object;

    // Force the correct set of flags for finding individuals.
    flags.searchIndividual = true;
    flags.searchCommunity  = false;
    flags.searchOU         = false;
    flags.searchName       = true;
    flags.sort             = 'ID';
    flags.inAll            = inAll;
    flags.isIn             = interestArray;

    iterator  = rb.contact.get([], flags);
    memIt     = new MemberIt(iterator);
    return memIt;
}

/********************************************************************************
Add 'Interest' related methods to the Method.
*********************************************************************************/
/********************************************************************************
rb$mem$interests
Return an array of 'interests. Do this by getting an array of membership 
information for the ID specified in the member object, then removing all those
with a name that does NOT start with teh rb$int$ prefix.
********************************************************************************/
Member.prototype.getInterests = function () {
var communities = rb.contact.memberOf(this.ID);

   var interests = new Array;

   for (var i=0;i<communities.length;i++) {
       if ((communities[i].type=='COMMUNITY') && (communities[i].name.indexOf('rb$int$'))>=0) {
           var newInterest = new Interest(communities[i]);
           interests[communities[i].ID] = newInterest;
       }
   }
   return interests;
}

/********************************************************************************
rb$mem$addInterest
Add an interest to this member.
********************************************************************************/
Member.prototype.addInterest = function (interestID) {
   return rb.contact.link(this.ID, interestID);
}

/********************************************************************************
rb$mem$remInterest
Add an interest to this member.
********************************************************************************/
Member.prototype.remInterest = function (interestID) {
   return rb.contact.unlink(this.ID, interestID);
}

