/*
 * MessageGroup
 * Constructor for a newsgroup.
 *
 * Takes a single parameter name which is non-empty, non-null is
 * used to set the group name that this instance will operate on.
 *
 */ 
var MessageGroup = function (name) {
  this.name   = "";
  this.exists = false;
  if (name)
    this.name = name;
}

/*
 * MessageGroup.load()
 *
 * Verifies the existance of the message group, sets internal state
 * about the group (first, last properties), and returns true
 * if the group exists within the message store. 
 */

MessageGroup.prototype.load = function () {
  var group = rb.message.group.groupInfo(this.name);

  this.exists = (group != null);

  if (this.exists) {
    this.first = group.first;
    this.last  = group.last;
    this.description = group.description;
    this.exists = true;
  }
  return false;
}

/*
 * loadBoardNames - private
 * Simple method that loads the list of existing message boards 
 * exactly once. This is useful because at the moment the message
 * template does not provide a means of checking the existence or
 * state of a single message board.
 *
 * The 'rb$ng$boardList' object is an associative name array
 * where each newsgroup is indexed by its name.
*/
MessageGroup.loadBoardNames = function() {
  
  if (MessageGroup.rb$ng$boardList==null) {
    MessageGroup.rb$ng$boardList = new Object;
    var groups = rb.message.group.listGroups();
    for (var i=0;groups != null && i<groups.length;i++) {
      MessageGroup.rb$ng$boardList[groups[i]] = {name:groups[i]};
    }
  }
}


MessageGroup.getStatus = function(name) {
  MessageGroup.loadBoardNames();
  var res = (name==null ? null : MessageGroup.rb$ng$boardList[name]);
  return res;
}


MessageGroup.listGroups = function() {
    MessageGroup.loadBoardNames();
    var res = MessageGroup.rb$ng$boardList;
    return res;
}

/*
 * MessageGroup.create()
 * Creates a new message group within the message store, with a 
 * name matching the current name property of this object
 *
 * Returns:
 *   Groupname on sucess,
 *   null on failure
 *
 */
MessageGroup.prototype.create = function(description) {
var res = null;

  if (MessageGroup.getStatus(this.name)==null) {
    // Doesn't exist so check the name - is it valid?
    if (this.name) {
      if ((this.name).search(/[^a-z0-9]/)>=0) {
        // Illegal name- return.               
        rb.debug.write("Illegal Name :'"+this.name+"'<br>\n");
        return null;
      }
      res = rb.message.group.createGroup(this.name, description);
    }
    if (!res)
      return null;

    MessageGroup.rb$ng$boardList[this.name] = {name:  this.name};
  }
  return res;
}
 
/*
 * MessageGroup.remove()
 * Removes the current group and all its articles
 * from the message store.
 *
 * Returns:
 *   Groupname on sucess,
 *   null on failure
 *
 */
MessageGroup.prototype.remove = function() {
  var res = null;
  if (this.name) {
    res = rb.message.group.removeGroup(this.name);
  }
  if (!res) {
    return null;
  }
  return(res);
}
  
/* MessageGroup.get()
 * 
 * Retrieves the headers of all articles within the message store for this group
 * 
 * populates the following properties with message objects:
 *	MessageGroup.article[]:
 *		numerically indexed array of article headers, in
 *		same order of message store (note that this
 *		isnt guarateed to be the same as the chronological
 *		order of posting but will be fairly close).
 *
 *	MessageGroup.message[]:
 *		The same article objects, indexed by the unique message ID
 *		of the messages.
 *	
 *	MessageGroup.thread[]:
 *		Indexed by the message ID of the head article of each thread
 *	
 *	In addition to the properties documented in the message template, 
 *	the following properties are added to each article object by the
 *	get method:
 *	
 *	child[]:
 *		A numerically indexed array of article objects corresponding to
 *		each article in the thread which is a direct response to this
 *		one.
 *
 *	parent:
 *		The article to which this is a response.
 *
 *	parentIndex:
 *		The index within the parent's child[] array property of this article.
 *
 */
MessageGroup.prototype.get = function(first) {

  if (!this.exists)
     // Board does not exist - do *not* attempt to read it!
     return;

  var lastthread = null;
  if(first != null)
    var flags = {firstIndex:first}
  else
    var flags = {};

  var iter = rb.message.group.get(this.name, flags);
  
  // Create the new properties of this
  this.article = new Array();
  this.message = new Object();
  this.thread = new Object();
  
  // Populate up the article and message properties
  for(var i = 0; iter != null && iter.getNextRow(); i++){
    var article = new Object();
    this.message[iter.messageID] = article;
    this.article[i] = article;
    
    for (var p in iter)
      article[p] = iter[p];
    //rb.debug.write("Got article ["+i+"] = "+this.article[i].messageID+"<BR>\n");
    //rb.debug.write("Got message ["+iter.messageID+"] = "+this.message[iter.messageID].messageID+"<BR>\n");
    
    article.child = new Array();
    article.replies = 0;
  }
  
  //rb.debug.write("after iterator collect this.message = "+this.message.toSource()+"<BR>");
  
  //    rb.debug.write("get - Got "+this.article.length+" articles<BR>");
  //for(var i in this.article)
  //rb.debug.write("get - "+this.article[i].toSource()+" <BR>&nbsp;<BR>");
  
  // Now that we have all the articles, lets build the thread tree...
  //  (we cant do this until we have all articles in, as messages and 
  //   responses arent guaranteed to be in chronological order in the 
  //   iterator from message.get() above).
  for(var i = 0; i < this.article.length; i++){
    var article = this.article[i];
    MessageGroup.decomposeAddress(article);
    article.thread = article;
    
    if(article.references != null && article.references.length > 0){
      // Starting at the begining (oldest) end of the reference list and try to
      // work out if it exists.
      for(var r = 0; r < article.references.length; r++){
        var ref = article.references[r];
        if(this.message[ref] != null){
          // if we haven't already flagged it as a thread then do so
          if(!this.message[ref].threaded){
            this.thread[ref] = this.message[ref];
            rb.debug.write("new thread = "+ref+"<br>\n");
            if(lastthread != null){
              //			    rb.debug.write("setting this.thread["+ref+"].lastThread = "+lastthread.messageID+"<br>\n");
              //			    rb.debug.write("setting this.thread["+lastthread.messageID+"].nextThread = "+ref+"<br>\n");
              this.thread[ref].lastThread = lastthread;
              lastthread.nextThread = this.thread[ref];
            }
            lastthread = this.thread[ref];
            //			rb.debug.write("lastthread = "+lastthread.messageID+"<br>\n");
          }
          this.message[ref].threaded = true;
          article.thread = this.message[ref];
          break;
        }
      }
      // Find the immediately preceding parent article from the tail of the
      //  reference list, add us to the child property of this parent
      //  then set up our parent and parentIndex properties
      for(var r = article.references.length - 1 ; r >= 0; r--){
        var ref = article.references[r];
        if(article.thread == null && this.message[ref].thread != null)
          article.thread = this.message[ref].thread;
        if(this.message[ref] != null){
          this.message[ref].child.push(article);
          article.threaded = true;
          article.parent = this.message[ref];
          article.parentIndex = this.message[ref].child.length - 1;
          for (var p = article.parent; p != null; p = p.parent)
            p.replies++;
          break;
        }
      }
    }
    
    if(article.thread == article){
      this.thread[article.messageID] = article;
      ref = article.messageID;
      if(lastthread != null){
        //		rb.debug.write("setting this.thread["+ref+"].lastThread = "+lastthread.messageID+"<br>\n");
        //		rb.debug.write("setting this.thread["+lastthread.messageID+"].nextThread = "+ref+"<br>\n");
        this.thread[ref].lastThread = lastthread;
        lastthread.nextThread = this.thread[ref];
      }
      lastthread = article;
    }
    
    
    
  }
  //    rb.debug.write("after get this.thread = "+this.thread.toSource()+"<BR>");
  //    rb.debug.write("after get this.message = "+this.message.toSource()+"<BR>");
  
}

/* MessageGroup.setMessage()
 *
 * This method creates a new message object from the passed data object
 * If the passed data object contains a valid message ID, then the contents 
 * of that message will be merged with the input object 
 * (properties of the input object take precedence). 
 */

MessageGroup.prototype.setMessage = function(input){
  
   var out = new Object;

   if (input.messageID != null && input.messageID != ""){
     if (!this.exists)
       this.load();

     if (!this.exists) {
       rb.debug.write("Exit from MessageGroup.prototype.setMessage because group failed to load.<br>");
       return null;
     }

     var msgid = {messageID:input.messageID};
     out = rb.message.group.messageBody(msgid);
     rb.debug.write("msgid="+msgid+"<br>");
     // dumpObj(out, "oldMessage");
   }
   
   if(out == null)
     out = new Object();
   
   
   var baseData = [ 
     "subject",
     "from", 
     "name", 
     "email", 
     "body",
     ];
   
   var arrayData = [
     "groups",
     "references"
     ];
   
   for(prop in baseData){
     if(input[baseData[prop]] != null)
       out[baseData[prop]] = input[baseData[prop]];
   }
   for(prop in arrayData){
     var arr = null;
     if(input[arrayData[prop]] != null){
       if(typeof input[arrayData[prop]] == "object")
         out[arrayData[prop]] = input[arrayData[prop]];
       else if((arr = input[arrayData[prop]].split(",")).length > 1)
         out[arrayData[prop]] = arr;
       else
         out[arrayData[prop]] = [input[arrayData[prop]]];
     }
     else
       out[arrayData[prop]] = [];
   }
   
   return (this.current = out);
   
}

/*
 * MessageGroup.loadMessage()
 * 
 * Takes a message object containing at least a valid messageID 
 * property as input.
 *
 * Populates all other information in the object
 * (including message body), and returns the full object.
 *
 * Returns:
 *   a fully populated message object on sucess
 *   null on failure
 *
 * Other effects:
 *   If a message with the same unique ID already exists within the
 *   message, article, or thread properties of this MessageGroup, then
 *   the object referenced within those properties will also be fully
 *   populated.
 *
 */
MessageGroup.prototype.loadMessage = function (head){

  if (!this.exists)
    return null;

  var msg = rb.message.group.messageBody(head);
  
  // If we already had this message in our properties tree, then
  // fold in the new things we have found out about it and return
  // the local object, not the newly created one
  if (msg != null){
    if (this.message[msg.messageID] != null){
      for (var p in msg)
        this.message[msg.messageID][p] = msg[p];
      msg = this.message[msg.messageID];
    }
    else{
      this.message[msg.messageID] = msg;
    }
    if(msg.from != null){
      MessageGroup.decomposeAddress(msg);
    }
    msg.htmlBody = MessageGroup.htmlExpand(MessageGroup.wrap_text(msg.body));
    
  }   
  return(msg);
}

/*
* MessageGroup.postMessage()
*
* Posts a new message to the current message Group
*
* Returns:
*   message ID on success,
*   null on failure
*
*/
MessageGroup.prototype.postMessage = function (msg){
  
  if (!this.exists)
    return null;
  
  var from = "";
  
  if(msg == null)
    msg = this.current;
  if (msg.name != null && msg.name != "")
    from = msg.name + " ";

  if (msg.email != null && msg.email != "")
    from += "<"+msg.email+">";
  else 
    from += "<"+msg.from+">";

  msg.from = from;
  msg.body = msg.body.replace(/\r\n/g, "\n");

  // Check body for lines that are too long.
  var body = msg.body.split('\n');
  var changed = false;
  var lnwrapat = 950;

  for (var i=0;i<body.length;i++) {
     if (body[i].length>lnwrapat) {
        var str = body[i];
        var newline = '';

        // Too long - split.
        for (var j=0;j<str.length;j+=lnwrapat) {
           newline += str.slice(j,j+lnwrapat);
           newline += '\n';
        }
        changed = true;
        body[i] = newline;
     }
  }

  if (changed)
     msg.body = body.join('\n');

  return rb.message.group.postMessage(msg);
}

/*
 * MessageGroup.blankReply()
 *
 * Given an existing message, creates a new message object
 * with Subject:, References: headers which reflect the original
 * message, and a quoted body from the original message.
 *
 * Returns:
 *   Groupname on sucess,
 *   null on failure
 *
 */
MessageGroup.prototype.blankReply = function (orig){
  var msg = new Object();
  
  if(orig == null){
    orig = this.current;
  }
  
  for(var p in orig){
    if(p != "date")
      msg[p] = orig[p];
  }
  
  if(!orig || !orig.body || orig.body == "")
    return(null);
  else{
    if(msg.references == null)
      msg.references = new Array();
    msg.references.push(msg.messageID);
    if(!msg.subject.match(/Re:/))
      msg.subject = "Re: "+msg.subject;

    msg.body = '<div class="Forum">'+msg.from+' Wrote: \n<table><tr><td bgcolor="#ff0000" width="1"><img src="/images/spacer.gif" width="1"></td><td>'+msg.body+'</div></td></tr></table></div>\n';
if (false) {
    var bodytext = msg.body.split(/\r*\n/);
    msg.body = msg.from+" Wrote: \n";
    for(var i = 0; i < bodytext.length; i++){
      msg.body += "> "+bodytext[i]+"\n";
    }
    msg.body += "\n"
}
  }	
  return(this.current = msg);
}

/*
 * MessageGroup.threadNext()
 *
 * Given a valid threaded message, finds the next message which is
 * either a reply to this message, or a reply to one of it predicessors
 * if none exists (i.e. this is the last or only message on the thread 
 * then this method returns null).
 *
 * Returns:
 *   Message on sucess,
 *   null on failure
 *
 */
MessageGroup.prototype.threadNext = function (message){
  
  if(message != null){
    // Try the first child...
    if(message.child != null && message.child[0] != null)
      return(message.child[0]);
    else {
      while (message.parent != null && message.parentIndex != null){
        if(message.parent.child[message.parentIndex+1] != null)
          return(message.parent.child[message.parentIndex+1]); 
        else
          message = message.parent;
      }
    }
  }
}    


/*
 * MessageGroup.threadPrev()
 *
 * Given a valid threaded message, finds the previous message which was 
 * also a reply to the same parent.
 * If none exists (i.e. this is the first or only message in the thread 
 * then this method returns null).
 *
 * Returns:
 *   Message on sucess,
 *   null on failure
 *
 */
MessageGroup.prototype.threadPrev = function (message){
  
  if(message != null){
    if (message.parent != null && message.parentIndex != null){
      if(message.parentIndex > 0 && message.parent.child[message.parentIndex-1] != null){
        message = message.parent.child[message.parentIndex-1];
        while(message.child.length > 0)
          message = message.child[message.child.length-1];
        return(message); 
      }
      else
        return(message.parent);
    }
  }
}


/*
 * MessageGroup.decomposeAddress()
 *
 * Takes a message as its parameter.
 * Decomposes either of the two legal RFC822 sender address forms:
 *  "Full Name <user@dom.ain>", or
 *  "user@dom.ain (Full Name)"
 * in the message template provided 'from' property.
 *
 * Sets the message 'name' and 'email' properties if it is able
 * to parse out anything intelligible.
 *
 * Returns:
 *   void
 *
*/
MessageGroup.decomposeAddress = function(msg){
  if(msg.from){
    // Decode any wierd non 7bit encoding in the from header
    msg.from = MessageGroup.rfc2047Decode(msg.from);
    // Kill any existing name and from properties 
    msg.name = msg.email = null;
    // Build an array of possible matches
    comp = new Array();
    // user@dom.ain (Full Name) case...
    comp[0] = msg.from.match(/^([^@]+@[a-zA-Z0-9\-\.]+)\s*\("*([^"\)]*)"*\)$/);
      // we want subscript 1 to be name, subscript 2 to be email
      if(comp[0] != null){
        var tmp = comp[0][1];
        comp[0][1] = comp[0][2];
        comp[0][2] = tmp;
      }
      // Full Name <user@dom.ain> case...
      comp[1] = msg.from.match(/"*([^"]+)"*\s+<([^>]*).*/);
        for(var c in comp){
          if(comp[c] != null && comp[c].length == 3){
            msg.name = comp[c][1];
            msg.email = comp[c][2];
          }
        }
        // didnt get anything, set email to be naked from header
        if(msg.name == null){
          msg.email = msg.from;
          msg.name = msg.from;
        }
  }
}


/*
* MessageGroup.htmlEscape()
*
* Takes a string as a parameter.
* Turns the string into legal, literal HTML
* by escaping a conservative range of special HTML characters
* Note that whilst this is superficially the same as JavaScript escape(),
* it only escapes a far more limited number of characters to produce
* legal HTML.
*
* Returns:
*   String HTML output
*
*/
MessageGroup.htmlEscape = function(str){
  var tab = [ 
    ["&", "&amp;"], 
    ["<", "&lt;"], 
    [">", "&gt;"], 
    ["\"", "&quot;"] ]; 
  
  for (var seq in tab){
    var r = new RegExp(tab[seq][0], "gi");
    str = str.replace(r, tab[seq][1]);
  }
  return(str);
}

MessageGroup.quoteEscape = function(str){
    str = (str).replace(/\"/, '\\"');
}

MessageGroup.htmlExpand = function(str){
  
  // Look for something which may be an http://something/fred.foo?n=1.. type url
  str = str.replace(/(http:\/\/[a-zA-Z0-9\/_\-\.\%\~]*)(\?[a-zA-Z0-9\/_\-\%=\.]*[a-zA-Z0-9\/_\-\%=])*((\&amp;)[a-zA-Z0-9\/_\-\.\%=]*[a-zA-Z0-9\/_\-\%])*\.*/g, '<A HREF="$1$2$3" class="message">$1</A>').replace(/(<A HREF="[^"]*)&amp;([^"]*")/, "$1\&$2");
  
  // Look for an e-mail address
  str = str.replace(/(\&lt;|\s)([a-z0-9A-Z\._\-]+@[a-z0-9A-Z\-]+\.[a-z0-9A-Z-\.]+)(&gt;|\s)/ig, '$1<A HREF="mailto:$2" class="message">$2</A>$3');
  
  return(str);
}



/*
* MessageGroup.removeMessage()
*
* Takes an article object as a parameter.
* Attempts to remove that article from the message store.
*
* Returns:
*   The previous article in the thread if the remove suceeds.
*
*/
MessageGroup.prototype.removeMessage = function(article){
  if(article == null)
    article = this.current;
  
  var dead = article;
  var prev = this.threadPrev(article);
  
  if (prev == null){
    for(var i = 0; i < this.article.length; i++)
      if(this.article[i] != null && this.article[i].messageID != article.messageID){
        prev = this.article[i];
        break;
      }
  }
  var res;

  try {res=rb.message.group.removeMessage(article)} catch (ev) {;}

  if(res != null){
    rb.debug.write("after remove, doing a re-load<BR>\n");
    
    // re-load the newsgroup
    this.get();
    // This is tricky...
    // this.get cleans out the old properties, but prev is a reference to an
    // object in the old properties which keeps them all alive and points to
    // an out of date structure. We re-load it by it's message ID from the new
    // version.
    if(prev != null){
      prev = this.message[prev.messageID];
    }
    
    return(prev);
  }
  else{
    return(null);
  }
}

MessageGroup.rfc2047Decode = function(text){
  
  var d;
  while((d = text.match(/(.*)*=\?[A-Za-z0-9\-]*\?[Qq]\?([^\?]*)\?=(.*)*/i)) != null && d.length >= 3){
    var m;
    var encoded = d[2];
    while((m = encoded.match(/(.*)*=([0-9a-zA-Z][0-9a-zA-Z])(.*)*/)) != null && m.length > 3){
      encoded = m[1]+String.fromCharCode(parseInt("0x"+m[2]))+m[3];
    }
    text = d[1]+encoded+d[3];
  }
  
  return(text);
}

MessageGroup.wrap_text = function(str){
  var input = str.split(/[\r\n]/);
  var output = "";
  for(var i = 0; i < input.length; i++){
    rb.debug.write ("wrap_text: got string &quot;"+input[i]+"&quot;<br>\n");
    var replyline = (input[i].charAt(0) == ">");
    var threshold = (replyline)?79:75;
    if(input[i].length > threshold){
      rb.debug.write ("wrap_text: length = "+input[i].length+" (&gt;"+threshold+")<br>\n");
      var first = input[i].substr(0,threshold);
      var last = input[i].substr(threshold);
      var wrapline = last;
      var whitesplit = first.match(/(.*)\s+(.*)/);
      if(whitesplit != null && whitesplit.length == 3){
        rb.debug.write ("wrap_text: whitesplit = "+whitesplit.toSource()+"<br>\n");
        first = whitesplit[1];
        var prefix = "";
        var m = new Array;
        if((m = input[i].match(/^(>[>\s]*).*/)) != null)
          prefix = m[1];
        wrapline = prefix + whitesplit[2] + last;
      }
      input[i] = first;
      input.splice(i+1, 0, wrapline);
    }
    output += input[i]+'<br>\n';
  }
  return(output);
}

MessageGroup.wrap_textxxx = function(str){
  return '<table><tr><td width="5" bgcolor="#000000">&nbsp;</td><td>'+str+'</td></tr></table>';
}


