/*
*
* This 'treestruct' library offers an overlay onto contacts and
* catalogue templates for document management.
*
* CONCEPTS:
*
* A 'Person' is an individual in both the contact template, and in the
* catalogue template. The catalogue template stores a node in the
* '/People/uName' node to which items and data can be attached.
*
* A 'Role' is a community in the contact template, and node in the
* catalogue template. The catalogue template stores a node in the
* '/Role/uName' node to which items and data can be attached.
*
* A 'thingType' is an attempt to define a document type to the application.
* It relates to a class of item in the catalogue, and is marked is being a
* particular 'thing' by being inserted into a container.
* The container is an item beneath '/thingType' in the catalogue.
*
* 'Location', 'Document' and 'Event' are simply document heirarchies.
* The structure of all of the above heirarchies is defined by an XML
* file, by default named '/include/struct.xml'
*
* Each node has a predefined "Class" so that it is possible to descend
* the tree based on a list of classes, or a list of literal node names.
* The topmost nodes have a class the same as their node-name.
*
*
* $log: $
*
*/


Tree = function ()
{
	// constructor
}

// Each class of data has a top-level category node beneath
// which its data is stored. Here we validate the node's name
// To add a node, it must be added here.
//
Tree.valid_types = { thingType:1, Document:1, Location:1, Role:1,
						Event:1, People:1 };

Tree.valTopNode = function (type)
{
	if( Tree.valid_types[type] != 1 )
		return false;
	return true;
}

// Returns true if the supplied ID is a top-level thing.
//
Tree.isTopNode = function ( id )
{
	for( var i in Tree.valid_types )
	{
		if( id == Tree.getTopNode( i ) )
			return true;
	}
	return false;
}

// Each class of data has a top-level category node beneath
// which its data is stored. Here we get the catalogue ID if available,
// or create this node if not.
//
// The top-node is ALWAYS a catalogue thing.
//
Tree.getTopNode = function (type)
{
	// Check if we're being asked for a valid role...
	if( !Tree.valTopNode(type) )
		return -1;

	// Check whether we have a nodeID stored in the context template already:
	var sess = rb.page.session.read();
	if( typeof(sess.rb$nodecache)!="object")
		sess.rb$nodecache = new Object;
	if( sess.rb$nodecache[type]!=null )
		return sess.rb$nodecache[type];

	// Not cached, so search for the "Type" catalogue node.
	var Search={ idMkr:"N", ID:0, nextLevel:1, keyword:[type],
					searchInName:1 };

	var Iter = rb.catalogue.category.get(Search);
	if( Iter.getNextRow() )
	{
		sess.rb$nodecache[type] = Iter.nodeID;
		rb.page.session.write(sess);
		return Iter.nodeID;
	}

	// If we're here, then the node doesn't exist!
	var Node = { name:type, parentID:0,
					description:'Parent node for '+type+' Heirarchy' };

	var created = rb.catalogue.category.create( Node );
	if( created > 0 )
	{
		sess.rb$nodecache[type] = created;
		rb.page.session.write(sess);
	}
	else
	{
		rb.page.error("Cannot create top-level node<br>\n");
	}
	return created;
}

// Documents or "things" have predefined types - Here we convert
// the type-ID into a type Name, using the context as a cache
// if possible.
//
Tree.getThingType = function (cntID)
{
	// Check whether we have an ID stored in the context template already:
	var sess = rb.page.session.read();
	if( typeof(sess.rb$contcache)!="object")
		sess.rb$contcache = new Object;
	for( var i in sess.rb$contcache )
		if( sess.rb$contcache[i] == cntID )
			return i;

	var top = Tree.getTopNode("thingType");
    var Search={ idMkr:"N", ID:top, nextLevel:1, keyword:[],
                 searchInCode:1 };
 
    var Iter = rb.catalogue.item.get(Search);

	while(Iter.getNextRow())
	{
		sess.rb$contcache[Iter.code] = Iter.itemID;
		if( Iter.itemID == cntID )
		{
			rb.page.session.write(sess);
			return Iter.code;
		}
	}
	rb.page.session.write(sess);
	return -1;
}

// Documents or "things" have predefined types - Here we convert
// the type-name into a container ID, using the context as a cache
// if possible.
//
Tree.getContainerID = function (type)
{
	if( type == null )
		return -1;
	// Check whether we have an ID stored in the context template already:
	var sess = rb.page.session.read();
	if( typeof(sess.rb$contcache)!="object")
		sess.rb$contcache = new Object;
	if( sess.rb$contcache[type]!=null )
		return sess.rb$contcache[type];

	var top = Tree.getTopNode("thingType");
	var Search={ idMkr:"N", ID:top, nextLevel:1, keyword:[type],
				 searchInCode:1 };

	var Iter = rb.catalogue.item.get(Search);

	while(Iter.getNextRow())
	{
		sess.rb$contcache[Iter.code] = Iter.itemID;
		if( Iter.code == type )
		{
			rb.page.session.write(sess);
			return Iter.itemID;
		}
	}
	rb.page.session.write(sess);
	return -1;
}

// Similar to Tree.getPath, but returns the nodeInfo object, or null for
// failure.
//
Tree.getPathInfo = function ( path )
{
	var thisNode = Tree.getPath(path);
	if( thisNode[0] < 1 ) return null;

	if( thisNode[1] == "Community" ) {
		var res=rb.contact.communityInfo(thisNode[0]);
		if( res != null )
			return {nodeID:res.ID, name:res.name,
						description:res.description,
						classname:"Community", customData:res.customData };
	} else if( thisNode[1] == "OU" ) {
		var res=rb.contact.OUInfo(thisNode[0]);
		if( res != null )
			return {nodeID:res.ID, name:res.name,
						description:res.description,
						classname:"OU", customData:res.customData };
	} else if( thisNode[1] == "Individual" ) {
		var res=rb.contact.individualInfo(thisNode[0]);
		if( res != null )
			return {nodeID:res.ID, name:res.uName,
						description:res.description,
						classname:"Individual", customData:res.customData };
	} else {
		var Search={ idMkr:"N", ID:thisNode[0], keyword:[] };
		var Iter = rb.catalogue.category.get(Search);
		if( Iter.getNextRow() )
		{
			return { nodeID:Iter.nodeID, name:Iter.name,
					 	description:Iter.description,
					 	classname:Iter.nodeData, customData:null };
		}
	}
	return null;
}

// Returns an array of matching Nodes based on the supplied path and
// class. Returns -1 for an error condition.
//
Tree.getNodesByPath = function (path, classname)
{
	// rb.debug.write("getNodesByPath "+path+", "+classname+"<br>\n");
	var thisNode = Tree.getPath(path);
	if( thisNode[0] < 1 ) return -1;

	var pathC=Tree.getClassFromPath( path );
	var result = [];

	var ok = false;
	var Search = {hasNodeID:thisNode[0]};
	if( (!classname || classname==null || classname=="Community") &&
						Tree.validateClassPath( pathC+'/Community' ) ) {
		Search.searchCommunity=1;
		ok = true;
	}
	if( (!classname || classname==null || classname=="OU") &&
						Tree.validateClassPath( pathC+'/OU' ) ) {
		Search.searchOU=1;
		ok = true;
	}
	if( (!classname || classname==null || classname=="Individual") &&
						Tree.validateClassPath( pathC+'/Individual' ) ) {
		Search.searchIndividual=1;
		ok = true;
	}
	if( ok ) {
		var Iter=rb.contact.get([], Search);
		while( Iter.getNextRow() ) {
			if( Iter.type == "INDIVIDUAL" )
				result.push({nodeID:Iter.ID, name:Iter.uName,
						description:Iter.description,
						classname:Tree.contactType[Iter.type],
						customData:Iter.customData});
			else
				result.push({nodeID:Iter.ID, name:Iter.name,
						description:Iter.description,
						classname:Tree.contactType[Iter.type],
						customData:Iter.customData});
		}
	}

	if( classname || classname==null || ( classname!="Community" && classname!="OU" && classname!="Individual" ) ) {
		var Search={ idMkr:"N", ID:thisNode[0], nextLevel:1, keyword:[] };
		if( classname && classname != null )
		{
			Search.keyword=["%"+classname+"%"];
			Search.searchInData=1;
		}
		var Iter = rb.catalogue.category.get(Search);
	
		while( Iter.getNextRow() )
		{
			if( !classname || classname == null || Iter.nodeData == classname )
				result.push({nodeID:Iter.nodeID, name:Iter.name,
						 	description:Iter.description,
						 	classname:Iter.nodeData,
						 	customData:null});
		}
	}
	// rb.debug.write("getNodesByPath = "+result.toSource()+"<br>\n");

// Give getClassFromPath a helping hand...
	if( Tree.classesFromPaths == null )
		Tree.classesFromPaths = new Object;
	for( var i in result )
		Tree.classesFromPaths[path+"/"+result[i].name] = pathC+"/"+result[i].classname;

	return result;
}

// Returns an array of matching Items based on the supplied path-list and
// thing-Type. May optionally recurse through the catalogue nodes to
// find documents at all depths.
//
// Duplicate items are removed from the returned list.
//
Tree.getItemsByPathList = function (pathlist, thingType, recurse)
{
	if( !recurse )
		recurse = false;

	var unique=new Object;
	for( var i in pathlist )
	{
		var addlist=Tree.getItemsByPath(pathlist[i], thingType, recurse);
		for( var j in addlist )
			unique[addlist[j].itemID] = addlist[j];
	}
	var result=[];
	for( var i in unique )
		result.push(unique[i]);

	return result;
}

// Returns an array of matching Items based on the supplied path and
// thing-Type. May optionally recurse through the catalogue nodes to
// find documents at all depths. recurse has no meaning if the node
// in question is a contact.
//
// (Same as getItemsByPathList, except only searches one path)
//
Tree.getItemsByPath = function (path, thingType, recurse)
{
	var thisNode = Tree.getPath(path);
	if( thisNode[0] < 1 ) return [];
	if( thisNode[1] == "Community" || thisNode[1] == "OU" || thisNode[1] == "Individual" )
		var idMkr="C";
	else
		var idMkr="N";

	var result = [];
	var Search={ idMkr:idMkr, ID:thisNode[0], keyword:[] };
	if( recurse && idMkr == "N" )
		Search.allLevels=1;

	var cntID = null;
	if( typeof thingType == "string" && thingType.length > 0 )
		cntID = Tree.getContainerID(thingType);
	if( cntID == -1 )
		return [];

	// rb.debug.write("From path : "+path+"<br>\n");
	// rb.debug.write("Item search : "+Search.toSource()+"<br>\n");
	var Iter = rb.catalogue.item.get(Search);
	while( Iter.getNextRow() )
	{
		if( cntID == null || Iter.containerID == cntID )
			result.push({itemID:Iter.itemID, name:Iter.code, code:Iter.code,
							 description:Iter.description,
							 thingType:Tree.getThingType(Iter.containerID),
							 itemData:Iter.itemData,
							 fileRefs:Iter.fileRefs});
	}
	return result;
}

// Returns an array of matching Items based on the supplied path and
// item name/code.
//
Tree.getItemsByName = function (path, code)
{
	if( code == null || code == "" ) return [];

	var thisNode = Tree.getPath(path);
	if( thisNode[0] < 1 ) return [];
	if( thisNode[1] == "Community" || thisNode[1] == "OU" || thisNode[1] == "Individual" )
		var idMkr="C";
	else
		var idMkr="N";

	var result = [];
	var Search={ idMkr:idMkr, ID:thisNode[0], keyword:[code], searchInCode:1 };

	var Iter = rb.catalogue.item.get(Search);
	while( Iter.getNextRow() )
	{
		result.push({itemID:Iter.itemID, name:Iter.code, code:Iter.code,
						 description:Iter.description,
						 thingType:Tree.getThingType(Iter.containerID),
						 itemData:Iter.itemData,
						 fileRefs:Iter.fileRefs});
	}
	return result;
}

// As per getItemsByPath, but simply returns a count of documents.
//
// If no thingType is specified, this is a fairly efficient count,
// otherwise there is a nasty overhead.
//
Tree.cntItemsByPath = function (path, thingType, recurse)
{
	var thisNode = Tree.getPath(path);
	if( thisNode[0] < 1 ) return 0;
	if( thisNode[1] == "Community" || thisNode[1] == "OU" || thisNode[1] == "Individual" )
		var idMkr="C";
	else
		var idMkr="N";

	var count = 0;
	var Search={ idMkr:idMkr, ID:thisNode[0], keyword:[] };
	if( recurse && idMkr == "N" )
		Search.allLevels=1;

	var cntID = null;
	if( typeof thingType == "string" && thingType.length > 0 )
		cntID = Tree.getContainerID(thingType);
	if( cntID == -1 )
		return 0;

	if( cntID == null )
		return rb.catalogue.item.count(Search);

	var Iter = rb.catalogue.item.get(Search);
	while( Iter.getNextRow() )
		if( Iter.containerID == cntID )
			count++;

	return count;
}

// Returns information about a named path
// returns : [nodeID, classname, description, classpath ]
//
// This data is cached in context where possible for efficiency.
//
Tree.getPath = function (path)
{
	// rb.debug.write("getPath "+path+"<br>\n");
	if( typeof path != "string" ) return [-1,"","",""];
	if( path.charAt(0) != "/" )
		path = "/"+path;
	// rb.debug.write("getPath: "+path+"<br>\n");

	var pathArray = Tree.splitPath(path);
	var top = Tree.getTopNode(pathArray[0])
	if( top < 1 ) return [-1,"","",""];
	if( pathArray.length < 2 )
		return [top,pathArray[0],"","/"+pathArray[0]];

	var classPath = "/"+pathArray[0];

// See if this search is cached in context.
	var sess = rb.page.session.read();
	var bestLen=0;
	if( typeof(sess.rb$pathcache)!="object")
        sess.rb$pathcache = new Array;
	else
	{
// rb.debug.write("pathcache = "+sess.rb$pathcache.toSource()+"<br>\n");
		for( var i=0; i<sess.rb$pathcache.length; i++ ) {
			if( sess.rb$pathcache[i] == null ) continue;

			// 100% cache match ?
			if( sess.rb$pathcache[i][0] == path )
				return sess.rb$pathcache[i].slice(1);

			// Partial cache match ?
			if( sess.rb$pathcache[i][0]+"/" == path.slice(0,sess.rb$pathcache[i][0].length+1) && sess.rb$pathcache[i][0].length > bestLen ) {
				bestLen = sess.rb$pathcache[i][0].length;
				bestPath= sess.rb$pathcache[i][0];
				bestNode= sess.rb$pathcache[i][1];
				bestTClass= sess.rb$pathcache[i][2];
				bestClass= sess.rb$pathcache[i][4];
				bestParent= sess.rb$pathcache[i][5];
			}
		}
	}

// Was there any sort of cache-hit ?
	if( bestLen > 0 ) {
		pathArray=Tree.splitPath(path.slice(bestLen+1));
		var thisNode = bestNode;
		var classPath = bestClass;
		var thisClass = bestTClass;
		var thisParent = bestParent;
	} else {
		// Starting from the very top...
		var thisClass = pathArray[0];
		pathArray.shift();
		var thisNode = top;
		var thisParent = 0;
	}
	var thisDescr = "";

	outerGetPath:
	while( pathArray.length > 0 ) {
rb.debug.write("thisNode="+thisNode+", thisClass="+thisClass+" want "+pathArray[0]+"<br>\n");

// If our current node is a Contact thing, then query there, otherwise
// query catalogue
		if(thisClass=="Community"||thisClass=="Individual"||thisClass=="OU")
			idMkr="C";
		else
			idMkr="N";

// Not a contact thing
	// First check in the catalogue.
		var Search={ idMkr:idMkr, ID:thisNode, nextLevel:1,
						keyword:[pathArray[0]], searchInName:1 };

		var Iter = rb.catalogue.category.get(Search);
		while( Iter.getNextRow() )
		{
			if( idMkr == "C" && Iter.nodeID == thisParent )
				continue;
			thisParent = thisNode;
			thisNode = Iter.nodeID;
			thisDescr = Iter.description;
			thisClass = Iter.nodeData;
			classPath += "/"+thisClass;
			pathArray.shift();
			continue outerGetPath;
		}

	// If that fails, then check the various contacts types that are valid.
	// (Only bother if were currently on a catalogue node.)
		var Search={ hasNodeID:thisNode, searchName:1 };
		var ok=false;
		if( idMkr== "N" && Tree.validateClassPath( classPath+'/Community' ) ) {
			Search.searchCommunity=1;
			ok=true;
		}
		if( idMkr== "N" && Tree.validateClassPath( classPath+'/Individual' ) ) {
			Search.searchIndividual=1;
			ok=true;
		}
		if( idMkr== "N" && Tree.validateClassPath( classPath+'/OU' ) ) {
			Search.searchOU=1;
			ok=true;
		}
		if( ok ) {
		// rb.debug.write("Look in contacts : "+Search.toSource()+"<br>\n");
			var keyw=[];
			if( Search.searchIndividual == 1 ) {
				var Ind = rb.contact.individualInfo(-1,pathArray[0]);
				if( Ind != null ) {
					thisParent = thisNode;
					thisNode = Ind.ID;
					thisDescr = Ind.description;
					thisClass = "Individual";
					classPath += "/"+thisClass;
					pathArray.shift();
					continue outerGetPath;
				}
			}
			if( Search.searchCommunity || Search.searchOU ) {
				keyw=[pathArray[0]];
				var Iter = rb.contact.get(keyw, Search);
				while( Iter.getNextRow() ) {
					if( Iter.type =="INDIVIDUAL" && Iter.uName != pathArray[0] )
						continue;
					if( Iter.type !="INDIVIDUAL" && Iter.name != pathArray[0] )
						continue;
					thisParent = thisNode;
					thisNode = Iter.ID;
					thisDescr = Iter.description;
					thisClass = Tree.contactType[Iter.type];
					classPath += "/"+thisClass;
					pathArray.shift();
					continue outerGetPath;
				}
			}
		}
		return [-1,"","",""];
	}

// Cache a successful result.
	if( sess.rb$pathcache.length > Tree.cachelen )
		sess.rb$pathcache.shift();
	sess.rb$pathcache.push([path, thisNode, thisClass, thisDescr, classPath, thisParent])
	rb.page.session.write(sess);

// Give getClassFromPath a helping hand...
	if( Tree.classesFromPaths == null )
		Tree.classesFromPaths = new Object;
	Tree.classesFromPaths[path] = classPath;

	return [thisNode, thisClass, thisDescr, classPath, thisParent];
}

// Convert a literal pathname into a path of class-names.
// uses getPath, so the results are cached for us.
//
Tree.getClassFromPath = function (path)
{
	if( Tree.classesFromPaths == null )
		Tree.classesFromPaths = new Object;
	if( Tree.classesFromPaths[path] != null )
		return Tree.classesFromPaths[path];

	// First the simple case of - It the path 100% class already?
	if( Tree.validateClassPath( path ) ) {
		Tree.classesFromPaths[path]=path;
		return path;
	}

	// Then check equally simple case - Is the path 100% literal?
	var got = Tree.getPath( path );
	if( got[0] != -1 ) {
		Tree.classesFromPaths[path]=got[3];
		return got[3];
	}

	// Now we need to check partial class/partial literal results.
	// This can get complicated.
	var pathArray = Tree.splitPath( path );
	if( !Tree.valTopNode( pathArray[0] ) )
		return -1;

	var soFar = ["/"+pathArray[0]];
	for( var i=1; i<pathArray.length; i++ )
	{
		var tail = pathArray[i];
		tail = tail.replace(/^<(.*)>$/, "$1");

		for( var j=0; j<soFar.length; j++ )
		{
			// Is next bit a literal?
			if( Tree.getPath( soFar[j]+"/"+tail )[0] != -1)
			{
				soFar[j] += "/"+tail;
				continue;
			}
			// Is next bit a class?
			var res=Tree.getNodesByPath( soFar[j], tail );
			if( res.length == 0 )
			{
				// If nothing exists here, then we can only validate
				// based on the given structure from here on in.
				var classSoFar = Tree.getPath( soFar[j] )[3];
				classSoFar += "/"+pathArray.slice(i).join("/");
				if( Tree.validateClassPath( classSoFar ) )
					return classSoFar;

				soFar.splice(j,1);
				j--;
			} else if( res.length == 1 )
			{
				soFar[j] += "/"+tail;
			} else
			{
				var root = soFar[j];
				soFar.splice(j,1);
				for( var k in res )
					soFar.splice(j, 0, root+"/"+res[k].name);
				j += res.length - 1;
			}
		}
	}
	if( soFar.length == 0 )
		return -1;
	var res = Tree.getPath( soFar[0] )[3];
	Tree.classesFromPaths[path]=res;
	return res;
}

// Walks backward through category nodes to establish all parent node-paths
// for a given item.
//
// Returns an array of paths.
//
Tree.getItemParents = function (itemID)
{
	var result = [];
// Catalogue node parent-types.
	var Iter = rb.catalogue.category.get({idMkr:'I', ID:itemID});
	while( Iter.getNextRow() )
	{
		if( Iter.ID == 0 ) continue;
		if( Iter.parentID == 0 )
			result.push("/"+Iter.name);
		else
			result.push(Tree.getNodeParent(Iter.nodeID, Iter.parentID, Iter.nodeData)+"/"+Iter.name);
	}

// Contact node parent-types
	var Iter = rb.contact.get([],{hasItemID:itemID, searchCommunity:1,
										searchIndividual:1, searchOU:1});
	while( Iter.getNextRow() )
	{
		if( Iter.type == "INDIVIDUAL" )
			result.push(Tree.getNodeParent(Iter.ID, null, Tree.contactType[Iter.type] )+"/"+Iter.uName);
		else
			result.push(Tree.getNodeParent(Iter.ID, null, Tree.contactType[Iter.type] )+"/"+Iter.name);
	}

rb.debug.write("Item Parents for "+itemID+" = "+result.toSource()+"<br>\n");
	return result;
}

// Checks to see whether the specified user has any form of access to the
// specified document (Item/thing)
//
Tree.getUserItem = function (uname, itemID)
{
	if( uname == "" )
		return null;
	var perms = Tree.getUserAccess( uname, "read" );
	if( perms.length == 0 && uname != null)
		return null;

	var Item=rb.catalogue.item.get({idMkr:'I', ID:itemID})
	if( Item.getNextRow() )
	{
		thisThingType = Tree.getThingType(Item.containerID);
		var res = {itemID:Item.itemID, name:Item.code, code:Item.code,
					 description:Item.description,
					 thingType:thisThingType,
					 itemData:Item.itemData,
					 fileRefs:Item.fileRefs};
	} else
		return null;

	// Specific case - Don't care if we have access, just Gimme!
	if( uname == null ) return res;

	var paths = Tree.getItemParents( itemID );

	var readable = Tree.rb$checkPerms( paths, perms, false );
	if( readable.length != 0 )
		return res;
	return null;
}

// Walks backward through category nodes to establish all parent node-paths
// for a given node.
// This routine will scan the cache for each node traversed in case the
// required path is already known.
//
// Returns a path string.
// If nodeID is supplied, nodeclass must be supplied
// If parentID is supplied, parentMkr must be supplied
//
Tree.getNodeParent = function (nodeID, parentID, nodeclass, parentMkr)
{
rb.debug.write("getNodeParent called with "+nodeID+", "+parentID+", "+nodeclass+", "+parentMkr+"<br>\n");
	var sess = rb.page.session.read();

// Check whether the requested node is in the cache???
// nodeID and class must match.
	if( nodeID > 0 && sess.rb$pathcache != null ) {
		for( var i in sess.rb$pathcache ) {
			if( sess.rb$pathcache[i] == null ) continue;
			if( sess.rb$pathcache[i][1] == nodeID && sess.rb$pathcache[i][2] == nodeclass ) {
				fullPath = sess.rb$pathcache[i][0];
				return fullPath.replace(/^(.*)\/[^\/]*$/, "$1");
			}
		}
	}

// Node is not in the cache, and parent ID is not provided.
	if( nodeclass=="Community"||nodeclass=="OU"||nodeclass=="Individual")
		idMkr="C";
	else
		idMkr="N";

	if( !parentID || parentID == null )
	{
		var Iter = rb.catalogue.category.get({idMkr:idMkr, ID:nodeID});
		if( idMkr=="N" ){
			if( Iter.getNextRow() ) {
				parentID = Iter.parentID;
				parentMkr = "N";
			} else
				return "";

			if( parentID == 0 ) {
				if( Tree.isTopNode(nodeID) )
			// This is a top-level node, so return OK.
					return "/"+Iter.name;

			// This has a contact parent, so we need to find it!
				var Iter = rb.contact.get([],{hasNodeID:nodeID,
							searchOU:1, searchIndividual:1, searchCommunity:1});
				if( Iter.getNextRow() ) {
					parentID = Iter.ID;
					parentMkr = "C";
				} else
					return "";
			}
		} else {
			var parentID = 0;
			while( Iter.getNextRow() ) {
// We're going to have several nodes returned. Only one is the parent.
// Parent IFF the node's parentID != 0 || (parentID == 0 && node is at top)
				if( Iter.parentID == 0 && Tree.isTopNode(Iter.nodeID) )
					return "/"+Iter.name;
				if( Iter.parentID != 0 || Tree.isTopNode(Iter.nodeID) ) {
					parentID = Iter.nodeID;
					parentMkr = "N";
					break;
				}
			}
			if( parentID == 0 )
				return "";
		}
	}

// Check whether the node's parent is in the cache.
	if( sess.rb$pathcache != null ) {
		for( var i in sess.rb$pathcache ) {
			if( sess.rb$pathcache[i] == null )
				continue;
			var parentClass = sess.rb$pathcache[i][2];
			if( parentMkr=="N" && ( parentClass=="Community" || parentClass=="OU" || parentClass=="Individual" ))
				continue;
			if( parentMkr=="C" && parentClass!="Community" && parentClass!="OU" && parentClass!="Individual" )
				continue;

			if( sess.rb$pathcache[i][1] == parentID )
				return sess.rb$pathcache[i][0];
		}
	}

// Node parent not in cache, so fetch the parent.
	var par;
	if( parentMkr == "C" ) {
		if( (par=rb.contact.communityInfo( parentID )) != null )
			return (Tree.getNodeParent(parentID,null,'Community',null)+"/"+par.name);
		if( (par=rb.contact.OUInfo( parentID )) != null )
			return (Tree.getNodeParent(parentID,null,'OU',null)+"/"+par.name);
		if( (par=rb.contact.individualInfo( parentID )) != null )
			return (Tree.getNodeParent(parentID,null,'Individual',null)+"/"+par.name);
	} else {
		var Iter = rb.catalogue.category.get({idMkr:'N', ID:parentID});
		if( Iter.getNextRow() )
		{
			if( Iter.parentID>0 )
				return (Tree.getNodeParent(-1,Iter.parentID,Iter.nodeData,'N')+"/"+Iter.name);
			else {
				if( Tree.isTopNode(parentID) )
					return "/"+Iter.name;
				return (Tree.getNodeParent(parentID,null,Iter.nodeData,null)+"/"+Iter.name);
			}
		}
	}
	return "";
}

// An "intelligent" node creator. This routine understands the struct.xml
// file format, and will create mandatory child-nodes for a requested path.
//
// If a node already exists, then its ID is returned, and no action is
// taken.
//
Tree.makePath = function (path, classname, description)
{
	rb.debug.write("makePath: "+path+", "+classname+"<br>\n");

	var pathArray = Tree.splitPath(path);
	var pathAdd = pathArray.pop();
	var parent = "/"+pathArray.join("/");
	var thisNode = Tree.getPath(parent);
	if( thisNode[0] < 1 ) return -1;

	var newNodeID = Tree.getPath(path)[0];
	if( newNodeID > 0 )
		return newNodeID;

	// Get XML containing validation rules for this path...
	Tree.getXml();

	// And validate the path create request (We may need to create some
	// default "Literal" nodes for example)

	var classPath = Tree.getClassFromPath( parent );
	var classArray= Tree.splitPath( classPath+"/"+classname );

	var ttags = Tree.Xml.root.find("tree");
	var ttag = null;
	for( var i=0; i<ttags.length; i++ )
	{
		if( ttags[i].attributes.name == classArray[0] )
		{
			ttag = ttags[i];
			break;
		}
	}
	if( ttag == null ) return -1;
	var validated = "/"+classArray.shift();
	pathArray.shift();
	pathArray.push(pathAdd);

	while( classArray.length > 0 )
	{
		var nextTag=null;
		for( var i=0; i<ttag.length; i++ )
		{
			if( ttag[i].type != "tag" ) continue;

			if(  ttag[i].name == "var" && ttag[i].attributes['class'] == classArray[0] )
			{
				nextTag = ttag[i];
				break;
			}
			if(  ttag[i].name == "lit" && ttag[i].attributes['class'] == classArray[0] && ttag[i].attributes.name == pathArray[0] )
			{
				nextTag = ttag[i];
				break;
			}
		}
		if( nextTag == null )
		{
			rb.debug.write("Could not validate creation of "+validated+"/"+classArray[0]+"<br>\n");
			return -1;
		}
		ttag=nextTag;
		validated += "/"+classArray.shift();
		pathArray.shift();
	}

	var newNode = Tree.rb$makePathHelp( thisNode[0], path, pathAdd, classname, description, classPath+"/"+classname, thisNode[1]);
	if( newNode < 1 ) return -1;

	// Any literal children need creating ???
	for( var i=0; i<nextTag.length; i++ )
	{
		if( nextTag[i] == null ) continue;
		if( nextTag[i].type != "tag" || nextTag[i].name != "lit" ) continue;

        var tname = nextTag[i].attributes.name;
        var tclass= nextTag[i].attributes['class'];
        if( tname == null || tname == "" )
        {
            rb.page.write('&lt;lit&gt; tag must have "name" attribute!<br>');
            continue;
        }
        if( tclass == null || tclass == "" )
        {
            rb.page.write('&lt;lit&gt; tag must have "class" attribute!<br>');
            continue;
        }
        if( Tree.Classes[tclass] == null )
        {
            rb.page.write('Illegal "class" attribute '+tclass+'!<br>');
            continue;
        }
		rb.debug.write("Auto-creating literal child<br>\n");
        Tree.makePath(path+"/"+tname, tclass, Tree.Classes[tclass]);
	}

	return newNode;
}

// Helper function for makePath - keeps things fractionally more readable.
//
Tree.rb$makePathHelp = function (parNode, path, name, classname, description, fullClass, parClass)
{
	if( description == null ) description = "";
	var newNode = { name:name.slice(0,200), description:description.slice(0,900)};

	if( classname=="Individual" || classname=="Community" || classname=="OU" )
	{
		// Cannot create contact thing beneath a contact thing
		if( parClass=="Individual" || parClass=="Community" || parClass=="OU" )
			return -1;

		newNode.description = newNode.description.slice(0,200);
rb.debug.write("Creating in contacts ");
rb.debug.write("("+path+" as class "+classname+")<br>\n");
		newNode.customData={};
		if( classname == "Individual" )
			thisNode = rb.contact.createIndividual(newNode);
		if( classname == "OU" )
			thisNode = rb.contact.createOU(newNode);
		if( classname == "Community" )
			thisNode = rb.contact.createCommunity(newNode);

		if( thisNode < 1 ) return -1;

		var res = rb.contact.linkCatNode(thisNode, parNode);
		if( res < 1 ){
			rb.contact.remove(thisNode);
			return -1;
		}
	} else {
		if( parClass=="Individual" || parClass=="Community" || parClass=="OU" )
			newNode.parentID=0;
		else
			newNode.parentID=parNode;

rb.debug.write("Creating in catalogue ");
rb.debug.write("("+path+" as class "+classname+")<br>\n");
		newNode.nodeData=classname;
		newNode.name = newNode.name.slice(0,90);
		var thisNode = rb.catalogue.category.create(newNode);

		if( thisNode < 1 ) return -1;

		if( parClass=="Individual" || parClass=="Community" || parClass=="OU" )
		{
			var res = rb.contact.linkCatNode(parNode, thisNode);
			if( res < 1 ){
				rb.catalogue.category.remove(thisNode);
				return -1;
			}
		}
	}

// Cache a successful result.
	var sess = rb.page.session.read();
	if( typeof(sess.rb$pathcache)!="object")
        sess.rb$pathcache = new Array;
	if( sess.rb$pathcache.length > Tree.cachelen )
		sess.rb$pathcache.shift();
	sess.rb$pathcache.push([path, thisNode, classname, description, fullClass, parNode])
	rb.page.session.write(sess);
	return thisNode;
}

// Creates an item of a particular "thingType" using a supplied set of data,
// and attach the item to a list of nodes.
//
Tree.makeItem = function( thingType, obj, nodes )
{
	if( typeof nodes!="object" || nodes.constructor!=Array || nodes.length==0)
		return -1;
	var cntID = Tree.getContainerID( thingType );
	if( obj.code == null || obj.code == "" || cntID < 1 )
		return -1;

	if( !obj.itemData ) obj.itemData = new Object;
	if( !obj.fileRefs ) obj.fileRefs = [];

	if( obj.description == null ) obj.description = "";
	var item = rb.catalogue.item.create({code:obj.code.slice(0,90),
								description:obj.description.slice(0,90), containerID:cntID,
								itemData:obj.itemData, fileRefs:obj.fileRefs});
	if( item < 1 ) return -1;

	var links=[];
	var clinks=[];
	for( var i in nodes )
	{
		var par = Tree.getPath(nodes[i]);
		if( par[0] > 0 ){
			if( par[1]!="Community" && par[1]!="OU" && par[1]!="Individual")
				links.push(par[0]);
			else
				clinks.push(par[0]);
		} else
			rb.debug.write("Illegal link path? : "+nodes[i]+"<br>\n");
	}
	if( (links.length + clinks.length) != nodes.length )
	{
		rb.debug.write("Some of the requested links were illegal.<br>\n");
		rb.catalogue.item.remove(item);
		return -1;
	}
	for( var i in links )
	{
		if( rb.catalogue.category.linkItem( item, links[i] ) > 0 ) continue;

		// link failed - remove item
		rb.catalogue.item.remove(item);
        return -1;
	}
	for( var i in clinks )
	{
		if( rb.contact.linkCatItem( clinks[i], item ) > 0 ) continue;

		// link failed - remove item
		rb.catalogue.item.remove(item);
        return -1;
	}
	return item;
}

// Edits an item. Data supplied in the first parameter (object) is passed to
// rb.catalogue.item.modify.
//
// addnodes provides a list of nodes to link the item to.
// delnodes provides a list of nodes to unlink from.
//
// If all links are removed, the item is deleted and 0 is returned.
//
Tree.changeItem = function( obj, addnodes, rmnodes )
{
	if( obj.itemID == null )
		return -1;

	var upd = {itemID:obj.itemID};
	if( obj.code != null ) upd.code = obj.code.slice(0,90);
	if( obj.description != null ) upd.description = obj.description.slice(0,90);
	if( obj.itemData != null ) upd.itemData = obj.itemData;

rb.debug.write("Calling modify with "+upd.toSource()+"<br>\n");
	var item = rb.catalogue.item.modify(upd);
	if( item < 1 ) return -1;

	if( obj.fileRefs != null ) {
		rb.catalogue.item.modify({itemID:obj.itemID, fileRefs:obj.fileRefs});
	}


	if( addnodes==null || typeof addnodes != "object" || addnodes.constructor != Array )
		addnodes = [];
	if( rmnodes==null || typeof rmnodes != "object" || rmnodes.constructor != Array )
		rmnodes = [];
	if( addnodes.length == 0 && rmnodes.length == 0 )
		return item;

	var links=[];
	var clinks=[];
	var Iter = rb.catalogue.category.get({idMkr:'I', ID:item});
	while( Iter.getNextRow() )
		links.push(Iter.nodeID);
	var Iter = rb.contact.get([],{hasItemID:item, searchOU:1, searchCommunity:1,
									searchIndividual:1});
	while( Iter.getNextRow() )
		clinks.push(Iter.ID);

	// Clean from addnodes and already added nodes...
	for( var i=0; i<addnodes.length; i++ ) {
		addnodes[i] = Tree.getPath(addnodes[i]);
		if( addnodes[i][0] < 0 ) {
			addnodes.splice(i,1);
			i--;
			continue;
		}

		parClass=addnodes[i][1];
		if( parClass=="Individual" || parClass=="Community" || parClass=="OU" )
			var lks = clinks;
		else
			var lks = links;
		for( var j in lks ) {
			if( addnodes[i][0] == lks[j] ) {
				addnodes.splice(i,1);
				i--;
				break;
			}
		}
	}
	// Clean from rmnodes any non-existent nodes...
	var lrm = [];
	var clrm = [];
	for( var i=0; i<rmnodes.length; i++ ) {
		rmnodes[i] = Tree.getPath(rmnodes[i]);
		if( rmnodes[i][0] < 0 )
			continue;

		parClass=rmnodes[i][1];
		if( parClass=="Individual" || parClass=="Community" || parClass=="OU" ){
			var lks = clinks;
			var realrm = clrm;
		} else {
			var lks = links;
			var realrm = lrm;
		}

		for( var j in lks ) {
			if( rmnodes[i][0] == lks[j] ) {
				realrm.push(lks[j]);
				break;
			}
		}
	}
rb.debug.write("Current links = "+links.toSource()+"<br>\n");
rb.debug.write("Current clinks = "+clinks.toSource()+"<br>\n");
rb.debug.write("Add links = "+addnodes.toSource()+"<br>\n");
rb.debug.write("Del links = "+lrm.toSource()+"<br>\n");
rb.debug.write("Del clinks = "+clrm.toSource()+"<br>\n");

	for( var i in addnodes )
	{
		parClass=addnodes[i][1];
		if( parClass=="Individual" || parClass=="Community" || parClass=="OU" )
			rb.contact.linkCatItem( addnodes[i][0], item );
		else
			rb.catalogue.category.linkItem( item, addnodes[i][0] );
	}
	for( var i in lrm )
		rb.catalogue.category.unlinkItem( item, lrm[i] );
	for( var i in clrm )
		rb.contact.unlinkCatItem( clrm[i], item );

	if( rb.catalogue.category.count({idMkr:'I', ID:item}) > 0 )
		return item;
	if( rb.contact.count([],{hasItemID:item, searchOU:1, searchCommunity:1,
												searchIndividual:1 }) > 0 )
		return item;

// No links remain, so remove.
	rb.catalogue.item.remove(item);
	return 0;
}

// Validate a class-path against the XML structure.
//
Tree.validateClassPath = function (path)
{
	// rb.debug.write("validateClassPath: "+path+"<br>\n");

	if( Tree.validClassPaths == null )
		Tree.validClassPaths = new Object;
	if( Tree.validClassPaths[path] == true )
		return true;

	if( path=="" || path == "/" ) return true;

	// Get XML containing validation rules for this path...
	Tree.getXml();

	var classArray= Tree.splitPath( path );

	var ttags = Tree.Xml.root.find("tree");
	var ttag = null;
	for( var i=0; i<ttags.length; i++ )
	{
		if( ttags[i].attributes.name == classArray[0] )
		{
			ttag = ttags[i];
			break;
		}
	}
	if( ttag == null ) return false;

	while( classArray.length > 1 ) {
		classArray.shift();
		classArray[0] = classArray[0].replace(/^<(.*)>$/, "$1");

		var nextTag=null;
		for( var i=0; i<ttag.length; i++ ) {
			if( ttag[i].type != "tag" ) continue;
			if( ttag[i].name != "var" && ttag[i].name != "lit" ) continue;
			if( ttag[i].attributes['class'] == classArray[0] )
			{
				nextTag = ttag[i];
				break;
			}
			if( ttag[i].name=="lit" && ttag[i].attributes.name==classArray[0] )
			{
				nextTag = ttag[i];
				break;
			}
		}
		if( nextTag == null )
			return false;
		ttag=nextTag;
	}
	Tree.validClassPaths[path]=true;
	return true;
}

// Get valid classnames that can be created as a child of the specified path.
//
Tree.getValidClassesIn = function (path)
{
	var pathC = Tree.getClassFromPath(path);
	if( pathC == -1 )
		return [];

	var classArray= Tree.splitPath( pathC );

	Tree.getXml();
	var ttags = Tree.Xml.root.find("tree");
	var ttag = null;
	for( var i=0; i<ttags.length; i++ )
	{
		if( ttags[i].attributes.name == classArray[0] )
		{
			ttag = ttags[i];
			break;
		}
	}
	if( ttag == null ) return [];

	while( classArray.length > 1 ) {
		classArray.shift();

		var nextTag=null;
		for( var i in ttag ) {
			if( ttag[i].type != "tag" ) continue;
			if( ttag[i].name != "var" && ttag[i].name != "lit" ) continue;
			if( ttag[i].attributes['class'] == classArray[0] )
			{
				nextTag = ttag[i];
				break;
			}
			if( ttag[i].name=="lit" && ttag[i].attributes.name==classArray[0] )
			{
				nextTag = ttag[i];
				break;
			}
		}
		if( nextTag == null )
			return [];
		ttag=nextTag;
	}

	var result = [];
	for( var i=0; i<ttag.length; i++ ) {
		if( ttag[i].type!="tag" || (ttag[i].name!="var" && ttag[i].name!="lit"))
			continue;
		if( ttag[i].name=="lit" )
			result.push( ttag[i].attributes.name );
		result.push( ttag[i].attributes['class'] );
	}

	Tree.rb$uniqueArray( result );
	return result;
}

Tree.splitPath = function (path)
{
	if( path == null || typeof path != "string" ) {
		rb.debug.write("Tree.splitPath = "+path.toSource()+"<br>\n");
		return [];
	}
	var arr = path.split("/");
	if( arr[0] == "" )
		arr.shift();
	return arr;
}

Tree.getXml = function (filename)
{
	if( filename == null || filename == "" )
		filename = "/include/struct.xml";
	if( Tree.XmlFile == filename && Tree.Xml != null )
		return;

	Tree.XmlFile = filename;
	Tree.Xml = new XmlParser;
	var bin = new Binary;
	bin.load(filename);
	Tree.Xml.build(bin.toString());
	delete bin;

	Tree.Classes=new Object;
    var ctags = Tree.Xml.root.find("class",false);
    for( var i=0; i<ctags.length; i++ )
    {
        var stag = ctags[i].find("summary")[0];
        var stext="";
        if( stag != null )
            stext=stag.execute().bodytext();
        // rb.debug.write("Class: "+ctags[i].attributes.name+" = "+stext+"<br>\n");
		Tree.Classes[ctags[i].attributes.name]=stext;
    }
}

// Return a list of paths to which the user has access.
// Specify mode="write" for write access list
//
Tree.getUserAccess = function( uname, mode )
{
	if( uname == null || uname == "" )
		return [];
	if( Tree.userAccess == null )
		Tree.userAccess = new Object;
	if( mode==null || mode != "write" )
		mode="read";
	if( Tree.userAccess[uname+":"+mode] != null )
		return Tree.userAccess[uname+":"+mode];

	if( Tree.getPath( "/People/"+uname )[0] < 1 )
		return [];
	var aUser = rb.contact.individualInfo(-1,uname);

	if( aUser == null )
		return [];

	var access = new Array;
	var memOf = Tree.getMemberOf( aUser.ID );
	for( var i in memOf )
	{
		// The next test is not necessary - What are the chances
		// we would use ts$access where it doesn't belong?!?!
		// if( Tree.getPath( "/Role/"+memOf[i].uName )[0] < 1 )
		// 	continue;
		if( memOf[i].customData==null || memOf[i].customData.ts$access==null || typeof( memOf[i].customData.ts$access ) != "object" )
			continue;
		if( memOf[i].customData.ts$access.read != null && mode!="write")
			access = access.concat(memOf[i].customData.ts$access.read);
		if( memOf[i].customData.ts$access.write != null)
			access = access.concat(memOf[i].customData.ts$access.write);
	}


// Now we have the basic access list, we need to qualify all of these
// entries against the user's attachments (If any)

	var attach = Tree.getUserAttach( uname );
	var attachC = new Array;
	for( var i in attach )
	{
		attachC[i] = Tree.splitPath(Tree.getClassFromPath(attach[i]));
		attach[i] = Tree.splitPath(attach[i]);
	}

	var result = [];
	for( var i in access )
	{
		var a = Tree.splitPath(access[i]);  // Access path to be qualified
		var hits = [];
		for( var j in attach )
		{
			var b = attach[j];  // Attachment literal path
			var c = attachC[j]; // Attachment class-path
			var d = "";			// Qualified access path built here
			for( k=0; k<a.length; k++ )
			{
				// Ran out of attach path, or class match...
				if( b[k] == null )
					d += "/"+a[k];
				// Matched a wildcard class path
				else if( a[k] == '<'+c[k]+'>' )
					d += "/"+a[k];
				// Matched an exact path or class path
				else if( a[k] == b[k] || a[k] == c[k] )
					d += "/"+b[k];
				// No match... 2 possibilities
				else
				{
				// No match, but the rest of the path is literal.
				// only check if we've matched at least the top level.
					var test = d+"/"+a.slice(k).join("/");
					if( k>0 && Tree.getPath( test )[0] != -1 )
					{
						d=test;
						break;
					}
				// No match at-all (ie. failure).
					d=null;
					break;
				}
			}
			if( d != null )
				hits.push(d);
		}
		if( hits.length == 0 )
			result.push(access[i]);
		else
			result = result.concat(hits);
	}
	Tree.rb$uniqueArray( result );
	Tree.userAccess[uname+":"+mode] = result.concat();
	return result;
}

// Return a list of role-nodes that have read/write access
// (specified in "mode") to the selected path.
//
Tree.getRolesFor = function( path, mode )
{
	if( !mode || mode==null || mode !="write" )
		mode = "read";
rb.debug.write("getRolesFor : "+path+", "+mode+"<br>\n");

	roles = Tree.getNodesByPath( "/Role", "Community" );
	if( roles == -1 || roles.length == 0 )
		return [];
	pathC = Tree.getClassFromPath( path );
	if( pathC == -1 )
		return [];

	var result = [];
	var p = Tree.splitPath( path );
	var c = Tree.splitPath( pathC );
	for( var i in roles )
	{
		if( roles[i].customData==null || roles[i].customData.ts$access==null )
			continue;
		var access=[];
		if( roles[i].customData.ts$access.read != null && mode!="write")
			access=access.concat(roles[i].customData.ts$access.read);
		if( roles[i].customData.ts$access.write != null)
			access=access.concat(roles[i].customData.ts$access.write);
		if( access.length == 0 )
			continue;


		// Now see if our path matches at-all...
		for( var j in access )
		{
			var a = Tree.splitPath( access[j] );
			if( a.length > p.length )
				continue;
			for( var k=0; k<a.length; k++ )
			{
				if( a[k] == p[k] || a[k] == c[k] || a[k] == '<'+c[k]+'>' )
					continue;
				break;
			}
			if( k==a.length ) {
				result.push(roles[i]);
				break;
			}
		}
	}
	return result;
}

// Update the list of nodes with which a user is associated. This facility
// is typically used to "pin" a user to a location.
//
// This does not update the database, only the supplied member object.
// The results are validated as OK paths before being added.
//
Tree.setUserAttach = function( aMember, att )
{
	if( aMember == null )
		return null;
	if( aMember.customData == null || aMember.customData.constructor != Object)
		aMember.customData = new Object;
	Tree.rb$uniqueArray( att );

	var result = [];

	for( var i in att )
	{
		if( att[i].slice(0,5) == "/Role" || att[i].slice(0,7) == "/People" )
			continue;
		var cp = Tree.getClassFromPath(att[i]);
		if( cp != -1 )
			result.push(att[i]);
	}
	aMember.customData.ts$attach = result;
	return result;
}

// Returns a list of user attachments, automatically attaches a user to
// his /People/uName and /Role/uName nodes.
//
// The results are validated as OK paths before being returned.
//
Tree.getUserAttach = function( uname, classPath )
{
	if( uname == null || uname == "" )
		return [];
	if( typeof uname == 'object' )
		var aUser = uname;
	else
		var aUser = rb.contact.individualInfo(-1,uname);
	if( aUser == null || aUser.uName == null )
		return [];
	uname = aUser.uName;
	
	if( !classPath || classPath == "" )
		classPath = null;

	if( Tree.userAttach == null )
		Tree.userAttach = new Object;
	if( Tree.userAttach[uname+":"+classPath] != null )
		return Tree.userAttach[uname+":"+classPath];

	if( classPath != null && !Tree.validateClassPath(classPath))
		return [];

	if( Tree.getPath( "/People/"+uname )[0] < 1 )
		return [];

// A default attachment for all users.
	var src = ["/People/"+uname];

//
// This is REALLY nasty - We assume stuff about the member abstraction's
// behaviour. This is simpy because this call is made OFTEN, and we cannot
// afford to
//     a) assume member is loaded
//     b) spend the time converting a member nicely, when we don't care!
//
	//rb.debug.write("Got user data of : "+aUser.toSource()+"<br>\n");
	if( aUser.customData!=null && aUser.customData.userData!=null &&
				typeof( aUser.customData.userData.ts$attach ) == "object" &&
				aUser.customData.userData.ts$attach.constructor == Array )
		src = src.concat(aUser.customData.userData.ts$attach);

// A set of default attachments in /Role for each membership.
	if( classPath == null || classPath.slice(0,5) == "/Role" ) {
		var memOf = Tree.getMemberOf( aUser.ID );
		for( var i in memOf ) {
			if( Tree.getPath( "/Role/"+memOf[i].uName )[0] < 1 )
				continue;
			src.push("/Role/"+memOf[i].uName);
		}
	}
	Tree.rb$uniqueArray( src );

// Validate all of the results against the proscribed structure
// just in case we have a cheat somewhere!
	var result = [];
	if( classPath!=null ) {
		for( var i in src ) {
			var cp = Tree.getClassFromPath(src[i]);
			if( cp == -1 ) continue;
			if( cp.length >= classPath.length && cp.slice(0,classPath.length) == classPath )
				result.push(src[i]);
		}
	}
	else
		result = src;

	Tree.userAttach[uname+":"+classPath] = result.concat();
	return result;
}

// This returns a list of paths:
// a) readable by a user based upon attachments and role (default)
// b) readable by the user based on their role
// c) writeable by the user
// Result set is specified by choosing "read", "role" or "write" as the qType
//
// Parameters are:
//	user name
//  classPath (or class/literal mix) to limit results
//  allrows BOOL
//
// "allrows" if true will cause all possible results to be returned,
// if "allrows" is false, the results are limited to the first level
// of path where a choice becomes available.
//
Tree.getUserPath = function( uname, classPath, allrows, qtype )
{
var dat = new Date();
	// rb.debug.write("getUserPath: "+classPath+","+allrows+","+qtype+"<br>\n");
	if( uname == null || uname == "" )
		return [];
	var pathArray = Tree.splitPath(classPath);
	if( !Tree.valTopNode(pathArray[0]) )
		return [];

	var listSoFar = ["/"+pathArray[0]];
	var extras = [];
	var depthSoFar = 0;
	var userAttach = null;

	if( !qtype || qtype == null || (qtype != "role" && qtype != "write" ))
		qtype="read";

	var perms = null;
	var roleAttach;
	if( qtype == "write" )
		perms = roleAttach = Tree.getUserAccess( uname, "write" );
	else
		roleAttach = Tree.getUserAccess( uname, "read" );

	Tree.getXml();

	while( ++depthSoFar < pathArray.length )
	{
// rb.debug.write("getUserPath timeSoFar = "+((new Date()-dat)/1000)+"s<br>\n");
		if( listSoFar.length == 0 )
			break;
		if( listSoFar.length > 1 )
		{
			Tree.rb$uniqueArray(listSoFar);
			if( (!allrows || allrows == false ) )
			{
				Tree.rb$checkPerms(listSoFar, perms, true);
				if( listSoFar.length > 1 )
					return listSoFar;
			}
		}
		var listAppend = [];
		for( var i=0; i<listSoFar.length; i++ )
		{
			var matchSoFar = listSoFar[i];
			var pathHere = matchSoFar+"/"+pathArray[depthSoFar];
			var classHere = pathArray[depthSoFar];
			// rb.debug.write("matchSoFar = "+matchSoFar+"\n");
			// rb.debug.write(", pathHere = "+pathHere+"\n");
			// rb.debug.write(", classHere = "+classHere+"<br>\n");

			if( classHere.match(/^<.*>$/) == null )
			{
				var res=Tree.getPath( pathHere );
				if( res[0] > 0 )
				{
					listSoFar[i] = pathHere;
					var res=Tree.rb$checkPerms([pathHere], roleAttach, true);
					if( res.length == 0 ) {
						listSoFar.splice(i,1);
						i--;
					}
					continue;
				}
			}
			if( classHere == "<>" )
				var nodes = Tree.getNodesByPath( matchSoFar );
			else if( !classHere || Tree.Classes[classHere] != null )
				var nodes = Tree.getNodesByPath( matchSoFar, classHere );
			else
				var nodes = [];
			if( nodes.length == 0 )
			{
			// If no child nodes match, but we have access to the current
			// path-so-far, and allrows = true, then keep
			// this as a result.
				if( allrows && allrows == true && Tree.rb$checkPerms([matchSoFar], roleAttach).length > 0 )
					extras=extras.concat([matchSoFar]);
				listSoFar.splice(i,1);
				i--;
				continue;
			}
			if( nodes.length == 1 )
			{
			// If there is only one child node, then we can fast-path it
			// by checking our access rights.
				listSoFar[i] += "/"+nodes[0].name;
				var res=Tree.rb$checkPerms([listSoFar[i]], roleAttach, true);
				if( res.length == 0 ) {
					listSoFar.splice(i,1);
					i--;
				}
				continue;
			}

// for "role" and "write" we need no more user attach data that we get
// from the userAccess list
			if( userAttach == null ) {
				if( qtype != "read" )
					userAttach = [];
				else
					userAttach = Tree.getUserAttach( uname );
			}

// Build a list of matching paths, reduced by attachments if appropriate.
			var shortList = [];
			var longList = [];
			// If we have multiple children, we can quickly reduce them
			// to a shortlist using our "userAttach" - This is
			// only useful in "read" mode, not "role" or "write"
			// but we build up our list of nodes here anyway
			for( var j in nodes )
			{
				var k = matchSoFar+"/"+nodes[j].name
				longList.push(k);
				if( qtype=="read" ) {
					for( var l in userAttach )
					{
						var m = userAttach[l];
						if( m.length < k.length )
							continue;
						if( m.slice(0,k.length) == k )
							shortList.push(k);
					}
				}
			}
// If we want role or write type permissions, then there is an additional list
// of access permissions based only on our role permissions.

			if( shortList.length==0 )
				shortList = longList;

// Make sure we have access to the resultant shortlist.
			// If we're going to drop off the end anyway, don't
			// waste time checking permissions
			if( depthSoFar+1 != pathArray.length )
				Tree.rb$checkPerms(shortList, roleAttach, true);

			listSoFar.splice(i,1);
			listAppend = listAppend.concat(shortList);
			i--;
		}
		listSoFar = listSoFar.concat(listAppend);
	}
	if( perms != null )
		listSoFar = Tree.rb$checkPerms(listSoFar.concat(extras), perms );
	else if( allrows || depthSoFar == pathArray.length )
		listSoFar = Tree.rb$checkPerms(listSoFar.concat(extras), roleAttach );
	else
		listSoFar.concat(extras);
	Tree.rb$uniqueArray(listSoFar);
rb.debug.write("getUserPath time = "+((new Date()-dat)/1000)+"s<br>\n");
	return listSoFar;
}

Tree.rb$checkPerms = function( result, perms, allowpartial )
{
	if( !perms || perms == null ) return result;

	if( allowpartial==null || allowpartial != true )
		allowpartial = false;

	var arr  = [];
	var arrC = [];
	var pass = [];
	for( var j in result )
	{
		arr[j]  = Tree.splitPath(result[j]);
		pass[j] = false;
		// Do a simple string match in case we have literal access!
		for( var i in perms )
		{
			var p = perms[i]+"/";
			var r = result[j]+"/";
			if(p.length<=r.length && p==r.slice(0,p.length)){
				pass[j] = true
				break;
			}
		}
		if( pass[j] == false )
			arrC[j] = Tree.splitPath(Tree.getClassFromPath(result[j]));
	}

	// Loop through the set of access rights.
	for( var i in perms )
	{
		var a = Tree.splitPath(perms[i]);

		// If we have access to "/" then we have full access!
		if( a[0] == null || a[0] == "" )
			return result;

		// Walk through each of the paths we're checking.
		for( var j in arr )
		{
			// If this path is already OK, continue
			if( pass[j] ) continue;
			var b = arr[j];
			var c = arrC[j];
			var match = true;

			// Walk the length of this path until we get a mismatch.
			for( var k=0; k<a.length; k++ )
			{
				if( b[k] == null && allowpartial )
					break;
				if( b[k] == null || (a[k] != b[k] && a[k] != c[k] && a[k] != '<'+c[k]+'>') )
				{
					match = false;
					break;
				}
			}
			if( match ) pass[j]=true;
		}
	}
	var i=0;
	for( var j=0; j<result.length; j++, i++ )
		if( !pass[i] )
		{
			result.splice(j,1);
			j--;
		}
	return result;
}

Tree.rb$uniqueArray = function( arr )
{
	// Remove duplicates.
	arr.sort();
	for( var i=1; i< arr.length; i++ )
	{
		if( arr[i] == arr[i-1] )
		{
			arr.splice(i,1);
			i--;
		}
	}
	return arr;
}

Tree.sortArrayByName = function( arr )
{
	function rb$sortfunc(a, b) { 
		return ( a.name == b.name ? 0 : ( a.name < b.name ? -1 : 1 )); }
	arr.sort(rb$sortfunc);
}

// Same as rb.contact.memberOf, but caches for lifetime of page.
//
Tree.getMemberOf = function( uid )
{
	if( Tree.memberOfs == null )
		Tree.memberOfs = new Object;
	if( Tree.memberOfs[uid] == null )
		Tree.memberOfs[uid] = rb.contact.memberOf(uid);

	return Tree.memberOfs[uid];
}

// Given a member object this method links the related user node
// in the catalogue.
//
Tree.createUserNode = function( aMember )
{
	if( aMember.ID < 1 )
		return -1;

	var uname = aMember.uName;
	var stext = aMember.description;

	var pnode = Tree.getPath("/People/"+uname)[0];
	if( pnode < 1 ) {
		var topNode = Tree.getTopNode('People');
rb.debug.write("createUserNode linking "+aMember.uName+" to /People<br>\n");
		if(rb.contact.linkCatNode( aMember.ID, topNode ) < 1 )
			return -1;
		pnode = aMember.ID;
rb.debug.write("createUserNode linked "+aMember.uName+" OK<br>\n");
	}

	if( pnode > 0 ) {
		// Check that all literal children are created for this node.
		Tree.getXml();
		var nextTag = Tree.Xml.root.select('//tree[@name="People"]/var[@class="Individual"]/lit');
		for( var i in nextTag ) {
			Tree.makePath('/People/'+aMember.uName+'/'+nextTag[i].attributes.name, nextTag[i].attributes['class'], "" );
		}
	}

	// Now re-check for mandatory roles.
	var roles = Tree.getNodesByPath( "/Role", "Community" );
	for( var i in roles )
	{
		var com = rb.contact.communityInfo(-1,roles[i].name);
		if( com != null && com.customData!=null && com.customData.rb$isMandatory == true)
			rb.contact.link( aMember.ID, com.ID );
	}
	return pnode;
}

// Given a community ID,  this method links the related role node
// in the catalogue.
//
Tree.createRoleNode = function( comm )
{
	var res = rb.contact.communityInfo( comm );
	if( res == null )
		return -1;

	var rnode = Tree.getPath("/Role/"+res.uName)[0];
	if( rnode < 1 ) {
		var topNode = Tree.getTopNode('Role');
		if(rb.contact.linkCatNode( comm, topNode ) < 1 )
			return -1;
		rnode = comm;
	}
	return rnode;
}

// Default values
Tree.cachelen = 10;
Tree.Xml = null;
Tree.XmlFile = "";
Tree.contactType={INDIVIDUAL:"Individual", COMMUNITY:"Community", OU:"OU" };
