// Common functions to manage user profiles. Portfolios are stored in several ways:
// 1. The contact record (aka Member object) stores a set of publicity flags
//    indicating which fields should be shown.
// 2. A textual 'profile' description is stored in a separate catalogue item,
//    under the 'Profile' category. This is indexed by the uName of the
//    individual. Each individual can also have a 'file' reference
//    containing an image file to be shown to other users. This is reference
//    from the catalogue item.

// Constructor function.
function Profile(aMember) {
  
  // Set following flag when loadCatItem is called.
  this.catLoaded  = false;
  this.hasProfile = false;;
  this.files      = [];

  if (aMember) {
    this.member = aMember;

    if (aMember.customData.profile!=null)
      this.hasProfile = true;
    else 
      aMember.customData.profile = {};

    this.data = aMember.customData.profile;
  }
}

// Load the extended profile in the catalogue item.
Profile.prototype.loadCatItem = function() {
  if (!this.catLoaded) {
    if (this.member) {
      this.catItem = ResolvePath('/Profiles/'+this.member.uName, 'I');

      if (this.catItem==null)
        this.catItem = {code:this.member.uName, itemData:{member:{}, profile:{}}, stockData:'', fileRefs:[]};
      else {
        // Make sure everything we need is here.
        if (this.catItem.fileRefs == null) this.catItem.fileRefs=[];
        if (this.catItem.itemData == null) this.catItem.itemData={};
      }
    }
    this.catLoaded = true;
  }
  return this.catItem;
}

// Load a file 
Profile.prototype.loadFile = function(index) {
  if (index==null)
    index=0;

  if (this.files[index])
    return this.files[index];

  // Load the catalogue item
  var catItem = this.loadCatItem();

  // Is there a file with this reference index?
  if (catItem.fileRefs[index]!=null) {
    // Is such a file - try and load it.
    var it = rb.file.get({fileID:catItem.fileRefs[index].fileID}) ;

    if (it.getNextRow()) {
      this.files[index] = {fileID:it.fileID, owner:it.owner, name:it.name, description:it.description, fileDef:it.fileDef};
    }
  }
  return this.files[index] ? this.files[index].fileDef : null;
}


// Store file. Note *nothing* is written to the database until a Profile.save method is called.
Profile.prototype.storeFile = function(name, owner, description, fileData, index) {
  if (index==null)
    index=0;

  if (index>=10)
    // Index too high.
    return false;

  if (name==null)
    name='';
  
  if(description==null)
    description='';
  
  if (fileData==null)
    return false;

  // Ready to go.
  if (this.files[index]==null)
    this.files[index] = {};

  var fl         = this.files[index];

  fl.name        = name;
  fl.description = description;
  fl.owner       = owner;

  fl.fileDef     = fileData;
  fl.needSave    = true;
  return true;
}

// Prepare the catalogue item associated with a user. This copies some information
// from the member record so that we can later use these records as the ket way of
// searching the membership profiles.
var memberProperties = {uName:1, ID:1, first_name:1, initial:1, surname:1};

Profile.prototype.prepareCatItem = function() {
var catItem = this.loadCatItem();

  // 1. Copy the basic contact information into the itemData area.
  if (catItem.itemData.member==null)
    catItem.itemData.member = {};

  var memberCopy = catItem.itemData.member;

  for (var i in memberProperties) {
    if (this.member[i]!=null)
      memberCopy[i] = this.member[i];
  }
  
  // 2. The 'skills' array in the itemData (if present) must be serialised and stored
  //    in the 'description' field. The string we store is simply the set of codes
  //    separated by spaces.
  var skills = catItem.itemData.skills;
  var str    = '';
  var sep    = '';
  for (var i in skills) {
    if (str.length>95)
      break;
    str += (sep+i);
    sep  = ' ';
  }

  // Append the users name onto the description.
  str += (' '+this.member.name);

  catItem.description = str;

  // 3. Copy the itemData.keywords field so we can then search on it.
  catItem.keywords = catItem.itemData.keywords;

  // 4. Append the keywords with '$$ORG-ORGID $$ so we can search for organisations 
  //    from the catalogue record. Note: If the organisation changes then we have to 
  //    revisit the profile. Need to do this in the 'edit user details' screens.
  catItem.keywords += (' '+Profile.generateOrg(this.member.parent));
}
Profile.generateOrg = function(id) {
  return '##ORG##'+id+'###';
}

// Create a new profile. If this has already been loaded then do *not* create again.
Profile.prototype.create = function() {
  if (this.member && this.member.ID>0) {
    // OK - try and read the existing profile. If it already exists
    var oldItem = ResolvePath('/Profiles/'+this.member.uName, 'I');
    if (oldItem!=null)
      // Already has a profile
      return false;

    // If there are any file objects - then we need to create those first.
    var catItem = this.loadCatItem();

    for (var idx=0;idx<this.files.length && idx<10;idx++) {
      if (this.files[idx]!=null) {
        // Found a file. Create it.
        var fileID = rb.file.create(this.files[idx]);
        if (fileID>0)
          catItem.fileRefs[idx] = {fileOrder:0, fileID:fileID};
      }
    }
    
    this.prepareCatItem();

    this.catItem.itemID = MakeItem(catItem, '/Profiles', false);
    if (this.catItem.itemID==-1) {
      // Couldn't save.
      return false;
    }
    return this.member.save();
  }
  return false;
}

Profile.prototype.isPublic = function() {
  return (this.data!=null && this.data.publicProfile==true);
}

// Save an existing modified profile. Includes saving/creating any new files.
Profile.prototype.save = function() {
  if (this.member==null)
    return false;

  if (this.catLoaded==false)
    this.loadCatItem();
  
  if (this.catItem==null || this.catItem.itemID==null || this.catItem.itemID<0)
    return this.create();

  if (this.member && this.member.ID) {
    // If there are any file objects - then we need to create those first.
    var catItem = this.catItem;

    for (var idx=0;idx<this.files.length && idx<10;idx++) {
      if (this.files[idx]!=null) {
        var fl = this.files[idx];

        // New file?
        if (fl.fileID==null || fl.fileID<0) {
          // New file - create it.
          fl.fileID = rb.file.create(fl);
          if (fl.fileID<0)
            return false;

          this.catItem.fileRefs[idx] = {fileOrder:0, fileID:fl.fileID};
        }
        else if (fl.needSave)
          rb.file.modify(fl);

        delete fl.needSave;
      }
    }

    // Prepare the catalogue item before writing.
    
    // Finally - save the catalogue item.
    this.prepareCatItem();
    if (rb.catalogue.item.modify(this.catItem)<0)
      return false;

    // Finally - store the updates to the member component itself.
    rb.debug.trace(true);
    var res=this.member.save();
    rb.debug.trace(false);
  }
  return false;
}
/**********************************************************
Skill set/interests.
The profile takes an XML file that defines a set of known
skill sets. These are presented as a multi-choice selection
to the user as they edit their profile. The information
allows people to find people by skill.

This skill set is loaded in from a fixed file stored at
  /custom/skills.xml
If the file is *not* present then the skill section is 
not presented to the user.

Result is an object with two properties. First is data and
contains a set of properties named by the skills. Second is
the order - an array in file order from the XML.
**********************************************************/
Profile.loadSkillSet = function() {
var fileData = new Binary;
var result   = {index:{}, order:[]};
  
  if (fileData.load('/custom/skills.xml')) {
    // File loaded so process the XML.
    var aParser = new XmlParser;

    if (aParser.build(fileData)) {
      // XML is OK. Extract the skill set.
      var skills = aParser.root.select('/skill');

      for (var i=0;i<skills.length;i++) {
        if (skills[i].attributes.name!=null && skills[i].attributes.code!=null) {
          result.order.push({code:skills[i].attributes.code, name:skills[i].attributes.name});
          result.index[skills[i].attributes.code] = skills[i].attributes.name;
        }
      }
    }
  }
  return result;
}

// Load skills for specific profile. Are stored in the catalogue item. In this case is a 
// single object with a single property identifying each registered skill.
Profile.prototype.loadSkillSet = function() {
  var catItem = this.loadCatItem();

  if (catItem.itemData.skills==null)
    catItem.itemData.skills = {};

  return catItem.itemData.skills;
}


